SemwidgJS.ChartsMultipleDatasets = function() {

	this._addProperties([
		['header-index', 'static', 'default = last'],
		['data-indices', 'static', 'Comma-separated list of indices, default = last - 1'],
		['data-labels', 'query', 'Comma-separated list of labels'],
		['colors', 'static', 'Comma-separated list of colors: (#FF00FF, #00FFFF) or a list of mappings: (mylabel: #FF00FF, default: #00FFFF)']
	]);
};

SemwidgJS.ChartsMultipleDatasets.prototype = new SemwidgJS.SwWidget();

SemwidgJS.ChartsMultipleDatasets.prototype.chart = null;

SemwidgJS.ChartsMultipleDatasets.prototype._buildChart = function(type, data, options) {

	if (this.chart) {
		this.chart.destroy();
	}

	var html = '';

	html += '<div id="' + this.getId() + '_container" ';
	html += this._getDefaultHtmlAttributes();
	html += '></div>'

	this._setHtml(html);

	var elem = document.getElementById(this.getId() + '_container');
	var height = elem.clientHeight;
	var width = elem.clientWidth;

	if (height <= 0) {
		height = 300;
	}

	if (width <= 0) {
		width = 500;
	}

	var canvasHtml = '<canvas id="' + this.getId() + '_canvas" ' +
			'width="' + width + '" height="' + height + '"></canvas>';

	elem.innerHTML = canvasHtml;

	var canvas = document.getElementById(this.getId() + '_canvas');
	
	this.chart = new Chart(
		canvas.getContext('2d'), 
		{
			'type':	type,
			'data': data,
			'options': options
		}
	);

};

SemwidgJS.ChartsMultipleDatasets.prototype._getColor = function(index, label) {

	if (!SemwidgJS.Util.String.isEmpty(this.getColors())) {

		var colors = this.getColors().split('(').join('').split(')').join('').split(',');

		if (this.getColors().indexOf(':') > -1) {

			var colorObject = {};
			var c;
			for (var i = 0; i < colors.length; i++) {
				c = colors[i].split(':');
				if (c.length > 1) {
					colorObject[c[0].trim()] = c[1].trim();
				}
			}

			if (colorObject[label]) {
				return colorObject[label];
			}else if (colorObject['default']){
				return colorObject['default'];
			}

		}else {
			return colors[index % colors.length].trim();
		}

	}
	
	return '#' + SemwidgJS.Util.getColor(index);

};

SemwidgJS.ChartsMultipleDatasets.prototype._getData = function(datasetOptions) {

	var joinedValues = this.getValueJoinedValues();

	var mathParser = new SemwidgJS.Util.MathParser();

	var data = {labels: [], datasets: []};

	var i, j, k;

	var labels = this.getDatalabelsDefaultValue();
	if (!SemwidgJS.Util.String.isEmpty(labels)) {
		labels = labels.split(',');
	}

	for (i = 0; i < joinedValues.length; i++) {

		var dataIndices = [];
		if (!SemwidgJS.Util.String.isEmpty(this.getDataindices())) {
			var di = this.getDataindices();
			dataIndices = di.split(',');
			for (j = 0; j < dataIndices.length; j++) {
				dataIndices[j] = '' + dataIndices[j].trim().toLowerCase();
			}
		}else {
			dataIndices.push('last - 1');
		}

		var headers = [];
		var dataRows = [];

		for (k = 0; k < dataIndices.length; k++) {
			dataRows.push([]);
		}

		var jv = joinedValues[i][0];

		var max = jv.getVariableNames().length - 1;

		var headerIndex = this.getHeaderindex();
		if (SemwidgJS.Util.String.isEmpty(headerIndex)) {
			headerIndex = 'last';
		}

		if ((headerIndex).toLowerCase() != 'none') {
			headerIndex = mathParser.parse(('' + headerIndex).toLowerCase().replace('first', 0).replace('last', max));

			if (headerIndex > max) {
				console.warn('Header-Index out of bounds. Max value is ' + max + '.');
				headerIndex = max;
			}

			if (headerIndex < 0) {
				headerIndex = 0;
			}
		}

		for (k = 0; k < dataIndices.length; k++) {
			dataIndices[k] = mathParser.parse(('' + dataIndices[k]).toLowerCase().replace('first', 0).replace('last', max));
			if (dataIndices[k] > max) {
				console.warn('Data-Index out of bounds. Max value is ' + max + '.');
				dataIndices[k] = max;
			}

			if (dataIndices[k] < 0) {
				dataIndices[k] = 0;
			}
		}

		var result;
		for (j = 0; j < jv.getResults().length; j++) {
			result = jv.getResults()[j];

			if (('' + headerIndex).toLowerCase() != 'none') {
				headers.push(result[headerIndex].getValue());
			}

			for (k = 0; k < dataIndices.length; k++) {
				dataRows[k].push(result[dataIndices[k]].getValue());
			}
		}

		if (headers.length > 0) {
			data.labels = headers;
		}

		var label, color;
		var suffix = '';
		if (i > 0) {
			suffix = ' ' + (i + 1);
		}

		for (j = 0; j < dataRows.length; j++) {

			if (labels && labels.length > j) {
				label = labels[j].trim() + suffix;
			}else {
				label = 'Dataset ' + (j + 1) + suffix;
			}

			jIndex = ((dataRows.length - 1) * j) + i;
			
			color = this._getColor(jIndex, label);
			
			var dataset = {
				label: label,
				backgroundColor: color,
				borderColor: SemwidgJS.Util.shadeColor(color, -0.1),
				hoverBackgroundColor: SemwidgJS.Util.shadeColor(color, 0.2),
				hoverBorderColor: SemwidgJS.Util.shadeColor(color, 0.1),
				pointBackgroundColor: color,
				pointBorderColor: SemwidgJS.Util.shadeColor(color, -0.1),
				pointHoverBackgroundColor: SemwidgJS.Util.shadeColor(color, 0.2),
				pointHoverBorderColor: SemwidgJS.Util.shadeColor(color, 0.1),
				data: dataRows[j]
			};

			data.datasets.push(dataset);

			if (datasetOptions) {
				for (var name in datasetOptions) {
					dataset[name] = datasetOptions[name];
				}
			}
		}
	}

	return data;

};

