SemwidgJS.ChartsSingleDataset = function() {

	this._addProperties([
		['header-index', 'static', 'default = last'],
		['data-index', 'static', 'default = last - 1'],
		['show-legend', 'query', 'default = true'],
		['colors', 'static', 'Comma-separated list of colors: (#FF00FF, #00FFFF) or a list of mappings: (mylabel: #FF00FF, default: #00FFFF)']
	]);
};

SemwidgJS.ChartsSingleDataset.prototype = new SemwidgJS.SwWidget();

SemwidgJS.ChartsSingleDataset.prototype.chart = null;

SemwidgJS.ChartsSingleDataset.prototype._buildChart = function(type, data, options) {

	if (this.chart) {
		this.chart.destroy();
	}

	var showLegend = true;

	if (!SemwidgJS.Util.String.isEmpty(this.getShowlegendDefaultValue())) {
		var sl = this.getShowlegendDefaultValue().trim().toLowerCase();
		if (sl == 'true' || sl == 'yes' || sl == '1') {
			showLegend = true;
		}else {
			showLegend = false;
		}
	}

	var html = '';

	html += '<div id="' + this.getId() + '_container" ';
	html += this._getDefaultHtmlAttributes();
	html += '></div>'

	this._setHtml(html);

	var elem = document.getElementById(this.getId() + '_container');
	var height = elem.clientHeight;
	var width = elem.clientWidth;

	if (height <= 0) {
		height = 300;
	}

	if (width <= 0) {
		width = 500;
	}

	var canvasHtml = '<canvas id="' + this.getId() + '_canvas" ' +
			'width="' + width + '" height="' + height + '"></canvas>';

	elem.innerHTML = canvasHtml;

	var canvas = document.getElementById(this.getId() + '_canvas');
	
	this.chart = new Chart(
		canvas.getContext('2d'), 
		{
			'type':	type,
			'data': data,
			'options': options
		}
	);

};

SemwidgJS.ChartsSingleDataset.prototype._getColor = function(index, label) {

	if (!SemwidgJS.Util.String.isEmpty(this.getColors())) {

		var colors = this.getColors().split('(').join('').split(')').join('').split(',');

		if (this.getColors().indexOf(':') > -1) {

			var colorObject = {};
			var c;
			for (var i = 0; i < colors.length; i++) {
				c = colors[i].split(':');
				if (c.length > 1) {
					colorObject[c[0].trim()] = c[1].trim();
				}
			}
			
			if (colorObject[label]) {
				return colorObject[label];
			}else if (colorObject['default']){
				return colorObject['default'];
			}

		}else {
			return colors[index % colors.length].trim();
		}

	}
	
	return '#' + SemwidgJS.Util.getColor(index);

};

SemwidgJS.ChartsSingleDataset.prototype._getData = function() {

	var joinedValues = this.getValueJoinedValues();

	var mathParser = new SemwidgJS.Util.MathParser();

	var data = {labels: [], datasets: []};

	var i, j, k;

	for (i = 0; i < joinedValues.length; i++) {

		var headerIndex = this.getHeaderindex();
		if (SemwidgJS.Util.String.isEmpty(headerIndex)) {
			headerIndex = 'last';
		}

		var dataIndex = this.getDataindex();
		if (SemwidgJS.Util.String.isEmpty(dataIndex)) {
			dataIndex = 'last - 1';
		}

		var jv = joinedValues[i][0];

		var max = jv.getVariableNames().length - 1;

		if (headerIndex.toLowerCase() != 'none') {
			headerIndex = mathParser.parse(('' + headerIndex).toLowerCase().replace('first', 0).replace('last', max));

			if (headerIndex > max) {
				console.warn('Header-Index out of bounds. Max value is ' + max + '.');
				headerIndex = max;
			}

			if (headerIndex < 0) {
				headerIndex = 0;
			}
		}

		if (dataIndex.toLowerCase() != 'none') {
			dataIndex = mathParser.parse(('' + dataIndex).toLowerCase().replace('first', 0).replace('last', max));

			if (dataIndex > max) {
				console.warn('Header-Index out of bounds. Max value is ' + max + '.');
				dataIndex = max;
			}

			if (dataIndex < 0) {
				dataIndex = 0;
			}
		}

		var result;
		var dataObject = {data: [], backgroundColor: [], hoverBackgroundColor: []};
		var color;
		for (j = 0; j < jv.getResults().length; j++) {
			result = jv.getResults()[j];

			if (('' + headerIndex).toLowerCase() != 'none') {
				if (result[headerIndex]) {
					data.labels.push(result[headerIndex].getValue());
				}
			}else {
				data.labels.push(null);
			}

			if (('' + dataIndex).toLowerCase() != 'none') {
				if (result[dataIndex]) {
					dataObject.data.push(result[dataIndex].getValue());
				}
			}else {
				dataObject.data.push(null);
			}

			jIndex = ((jv.getResults().length - 1) * j) + i;
			color = this._getColor(jIndex, data.labels[data.labels.length - 1]);
			dataObject.backgroundColor.push(color);
			dataObject.hoverBackgroundColor.push(SemwidgJS.Util.shadeColor(color, 0.2));
		}

		data.datasets.push(dataObject);

	}

	return data;

};

