/**
 * Config
 *
 * Reads and stores all data-sw-config elements
 */

SemwidgJS.Config = function() {	
	
};

SemwidgJS.Config.prototype = new SemwidgJS.EventDispatcher();

SemwidgJS.Config.prototype.clear = function() {
	var i;
	for (i = this.getProxies().length - 1; i >= 0; i--) {
		SemwidgJS.Util.destroy(this.getProxies()[i]);
	}
	this.proxies = null;

	for (i = this.getEndpoints().length - 1; i >= 0; i--) {
		SemwidgJS.Util.destroy(this.getEndpoints()[i]);
	}
	this.endpoints = null;

	for (i = this.getResources().length - 1; i >= 0; i--) {
		SemwidgJS.Util.destroy(this.getResources()[i]);
	}
	this.resources = null;

	for (i = this.getNamespaces().length - 1; i >= 0; i--) {
		SemwidgJS.Util.destroy(this.getNamespaces()[i]);
	}
	this.namespace = null;

	for (i = this.getMappings().length - 1; i >= 0; i--) {
		SemwidgJS.Util.destroy(this.getMappings()[i]);
	}
	this.mappings = null;
	
	SemwidgJS.getTemplateProvider().templates = null;
}

SemwidgJS.Config.prototype.updateConfig = function(node) {

	this.readProxyConfig(node);
	this.readEndpointConfig(node);
	this.readResourceConfig(node);
	this.readNamespaceConfig(node);
	this.readMappingConfig(node);

	// passes all TemplateDefinitions to TemplateProvider
	this.readTemplateDefinitions();
};

SemwidgJS.Config.prototype.readProxyConfig = function(node) {

	var proxyNodeList;

	if (node) {
		proxyNodeList = $(node).find('[data-sw-config="proxy"]');
	}else {
		proxyNodeList = $.find('[data-sw-config="proxy"]');
	}

	for (var i = 0; i < proxyNodeList.length; i++) {
		
		var proxyNode = proxyNodeList[i];
		var proxy = new SemwidgJS.Proxy();

		proxy.setId($(proxyNode).data('swp-id'));
		proxy.setUrl($(proxyNode).data('swp-url'));

		proxy._setNode(proxyNode);

		this.addProxy(proxy);
	}
};

SemwidgJS.Config.prototype.readEndpointConfig = function(node) {

	var endpointNodeList;

	if (node) {
		endpointNodeList = $(node).find('[data-sw-config="endpoint"]');
	}else {
		endpointNodeList = $.find('[data-sw-config="endpoint"]');
	}

	for (var i = 0; i < endpointNodeList.length; i++) {
		
		var endpointNode = endpointNodeList[i];
		var endpoint = new SemwidgJS.Endpoint();

		endpoint.setId($(endpointNode).data('swp-id'));
		endpoint.setUrl($(endpointNode).data('swp-url'));
		endpoint.setDefaultGraphUri($(endpointNode).data('swp-default-graph-uri'));
		endpoint.setDefaultLang($(endpointNode).data('swp-default-lang'));
		endpoint.setProxy($(endpointNode).data('swp-proxy'));
		endpoint.setDefault($(endpointNode).data('swp-default'));
		endpoint.setHttpMethod($(endpointNode).data('swp-http-method'));

		endpoint._setNode(endpointNode);

		this.addEndpoint(endpoint);
	}
};

SemwidgJS.Config.prototype.readResourceConfig = function(node) {

	var resourceNodeList;

	if (node) {
		resourceNodeList = $(node).find('[data-sw-config="resource"]');
	}else {
		resourceNodeList = $.find('[data-sw-config="resource"]');
	}

	for (var i = 0; i < resourceNodeList.length; i++) {
		
		var resourceNode = resourceNodeList[i];
		var resource = new SemwidgJS.Resource();

		resource.setId($(resourceNode).data('swp-id'));
		resource.setUri($(resourceNode).data('swp-uri'));
		resource.setEndpoint($(resourceNode).data('swp-endpoint'));

		resource._setNode(resourceNode);

		this.addResource(resource);
	}
};

SemwidgJS.Config.prototype.readNamespaceConfig = function(node) {

	this.buildPredefinedNamespaces();

	var namespaceNodeList;

	if (node) {
		namespaceNodeList = $(node).find('[data-sw-config="namespace"]');
	}else {
		namespaceNodeList = $.find('[data-sw-config="namespace"]');
	}

	for (var i = 0; i < namespaceNodeList.length; i++) {
		
		var namespaceNode = namespaceNodeList[i];
		var namespace = new SemwidgJS.Namespace();

		namespace.setId($(namespaceNode).data('swp-id'));
		namespace.setUri($(namespaceNode).data('swp-uri'));

		namespace._setNode(namespaceNode);

		this.addNamespace(namespace);
	}
};

SemwidgJS.Config.prototype.readMappingConfig = function(node) {

	var mappingNodeList;

	if (node) {
		mappingNodeList = $(node).find('[data-sw-config="mapping"]');
	}else {
		mappingNodeList = $.find('[data-sw-config="mapping"]');
	}

	for (var i = 0; i < mappingNodeList.length; i++) {
		
		var mappingNode = mappingNodeList[i];
		var mapping = new SemwidgJS.Mapping();

		mapping.setId($(mappingNode).data('swp-id'));
		mapping.setValue($(mappingNode).data('swp-value'));

		mapping._setNode(mappingNode);

		this.addMapping(mapping);
	}
};

SemwidgJS.Config.prototype.readTemplateDefinitions = function() {

	var templateDefinitionNodeList = $.find('[data-sw-config="template"]');

	for (var i = 0; i < templateDefinitionNodeList.length; i++) {
		
		var templateDefinitionNode = templateDefinitionNodeList[i];
		SemwidgJS.getTemplateProvider().registerTemplateDefinition(templateDefinitionNode);

	}

};

SemwidgJS.Config.prototype.addProxy = function(proxy) {

	if (SemwidgJS.Util._checkIdProxy(proxy.getId())) {

		if (this.getProxies() === undefined) {
			this.proxies = [proxy];
		}else {
			this.getProxies().push(proxy);
		}

	}else {

		var prx = this.getProxy(proxy.getId()) 
		if (prx) {
			prx.setUrl(proxy.getUrl());

			prx._setNode(proxy._getNode());
			
		}else {
			this.getProxies().push(proxy);
		}
	}
};
	
SemwidgJS.Config.prototype.getProxy = function(id) {
	for (var i = 0; i < this.getProxies().length; i++) {
		if (this.getProxies()[i].getId() == id) {
			return this.getProxies()[i];
		}
	}
	return null;
};
	
SemwidgJS.Config.prototype.getProxies = function() {
	if (!this.proxies) {
		return this.proxies = [];
	}
	return this.proxies;
};

SemwidgJS.Config.prototype.addEndpoint = function(endpoint) {
	
	if (SemwidgJS.Util._checkIdEndpoint(endpoint.getId())) {

		if (this.getEndpoints() === undefined) {
			this.endpoints = [endpoint];
		}else {
			this.getEndpoints().push(endpoint);
		}

	}else {

		var end = this.getEndpoint(endpoint.getId()) 
		if (end) {
			end.setUrl(endpoint.getUrl());
			end.setDefaultGraphUri(endpoint.getDefaultGraphUri());
			end.setDefaultLang(endpoint.getDefaultLang());
			end.setProxy(endpoint.getProxy());
			end.setDefault(endpoint.getDefault());
			end.setHttpMethod(endpoint.getHttpMethod());

			end._setNode(endpoint._getNode());

		}else {
			this.getEndpoints().push(endpoint);
		}
	
	}
};
	
SemwidgJS.Config.prototype.getEndpoint = function(id) {
	for (var i = 0; i < this.getEndpoints().length; i++) {
		if (this.getEndpoints()[i].getId() == id) {
			return this.getEndpoints()[i];
		}
	}
	return null;
};
	
SemwidgJS.Config.prototype.getEndpoints = function() {
	if (!this.endpoints) {
		this.endpoints = [];
	}
	return this.endpoints;
};

SemwidgJS.Config.prototype.addResource = function(resource) {
	
	if (SemwidgJS.Util._checkIdResource(resource.getId())) {

		if (this.getResources() === undefined) {
			this.resources = [resource];
		}else {
			this.getResources().push(resource);
		}

	}else {

		var res = this.getResource(resource.getId()) 
		if (res) {
			res.setUri(resource.getUri());
			res.setEndpoint(resource.getEndpoint());

			res._setNode(resource._getNode());
			
		}else {
			this.getResources().push(resource);
		}
	
	}
};
	
SemwidgJS.Config.prototype.getResource = function(id) {
		for (var i = 0; i < this.getResources().length; i++) {
		if (this.getResources()[i].getId() == id) {
			return this.getResources()[i];
		}
	}
	return null;
};
	
SemwidgJS.Config.prototype.getResources = function() {
	if (!this.resources) {
		this.resources = [];
	}
	return this.resources;
};

SemwidgJS.Config.prototype.addNamespace = function(namespace) {
	
	if (SemwidgJS.Util._checkIdNamespace(namespace.getId())) {

		if (this.getNamespaces() === undefined) {
			this.namespaces = [namespace];
		}else {
			this.getNamespaces().push(namespace);
		}

	}else {

		var ns = this.getNamespace(namespace.getId()) 
		if (ns) {
			ns.setUri(namespace.getUri());

			ns._setNode(namespace._getNode());
			
		}else {
			this.getNamespaces().push(namespace);
		}

	}
};
	
SemwidgJS.Config.prototype.getNamespace = function(id) {
		for (var i = 0; i < this.getNamespaces().length; i++) {
		if (this.getNamespaces()[i].getId() == id) {
			return this.getNamespaces()[i];
		}
	}
	return null;
};
	
SemwidgJS.Config.prototype.getNamespaces = function() {
	if (!this.namespace) {
		this.namespace = [];
	}
	return this.namespace;
};

SemwidgJS.Config.prototype.addMapping = function(mapping) {
	
	if (SemwidgJS.Util._checkIdMapping(mapping.getId())) {

		if (this.getMappings() === undefined) {
			this.mappings = [mapping];
		}else {
			this.getMappings().push(mapping);
		}

	}else {

		var map = this.getMapping(mapping.getId()) 
		if (map) {
			map.setValue(mapping.getValue());

			map._setNode(mapping._getNode());
			
		}else {
			this.getMappings().push(mapping);
		}

	}
};
	
SemwidgJS.Config.prototype.getMapping = function(id) {
	for (var i = 0; i < this.getMappings().length; i++) {
		if (this.getMappings()[i].getId() == id) {
			return this.getMappings()[i];
		}
	}
	return null;
};
	
SemwidgJS.Config.prototype.getMappings = function() {
	if (!this.mappings) {
		this.mappings = [];
	}
	return this.mappings;
};


SemwidgJS.Config.prototype.buildPredefinedNamespaces = function() {

	var pna = SemwidgJS.Config._predefinedNamespacesArray;

	for (var i = 0; i < pna.length; i++) {
		
		var namespace = new SemwidgJS.Namespace();
		namespace.setId(pna[i][0]);
		namespace.setUri(pna[i][1]);
		
		this.addNamespace(namespace);
	}
};

SemwidgJS.Config._predefinedNamespacesArray = [
	['a', 'http://www.w3.org/2005/Atom'],
	['aapi', 'http://rdf.alchemyapi.com/rdf/v1/s/aapi-schema#'],
	['accom', 'http://purl.org/acco/ns#'],
	['activity', 'http://activitystrea.ms/spec/1.0/'],
	['address', 'http://schemas.talis.com/2005/address/schema#'],
	['admin', 'http://webns.net/mvcb/'],
	['amz', 'http://webservices.amazon.com/AWSECommerceService/2011-08-01'],
	['atom', 'http://atomowl.org/ontologies/atomrdf#'],
	['audio', 'http://purl.org/media/audio#'],
	['awol', 'http://bblfish.net/work/atom-owl/2006-06-06/#'],
	['aws', 'http://soap.amazon.com/'],
	['b3s', 'http://b3s.openlinksw.com/'],
	['batch', 'http://schemas.google.com/gdata/batch'],
	['bestbuy', 'http://remix.bestbuy.com/'],
	['bibo', 'http://purl.org/ontology/bibo/'],
	['bif', 'bif:'],
	['bio', 'http://purl.org/vocab/bio/0.1/'],
	['book', 'http://purl.org/NET/book/vocab#'],
	['bookmark', 'http://www.w3.org/2002/01/bookmark#'],
	['bugzilla', 'http://www.openlinksw.com/schemas/bugzilla#'],
	['buzz', 'http://schemas.google.com/buzz/2010'],
	['c', 'http://www.w3.org/2002/12/cal/icaltzd#'],
	['calais', 'http://s.opencalais.com/1/pred/'],
	['category', 'http://dbpedia.org/resource/Category:'],
	['cb', 'http://www.crunchbase.com/'],
	['cc', 'http://web.resource.org/cc/'],
	['cert', 'http://www.w3.org/ns/auth/cert#'],
	['cl', 'http://www.ebusiness-unibw.org/ontologies/consumerelectronics/v1#'],
	['cnet', 'http://api.cnet.com/restApi/v1.0/ns'],
	['Comps', 'http://www.zillow.com/static/xsd/Comps.xsd'],
	['contact', 'http://www.w3.org/2000/10/swap/pim/contact#'],
	['content', 'http://purl.org/rss/1.0/modules/content/'],
	['cp', 'http://schemas.openxmlformats.org/package/2006/metadata/core-properties'],
	['crosspost', 'http://purl.org/syndication/cross-posting'],
	['ctag', 'http://commontag.org/ns#'],
	['cv', 'http://purl.org/captsolo/resume-rdf/0.2/cv#'],
	['cvbase', 'http://purl.org/captsolo/resume-rdf/0.2/base#'],
	['cve', 'http://www.openlinksw.com/schemas/cve#'],
	['d', 'http://schemas.microsoft.com/ado/2007/08/dataservices'],
	['dawgt', 'http://www.w3.org/2001/sw/DataAccess/tests/test-dawg#'],
	['dbpedia', 'http://dbpedia.org/resource/'],
	['dbpedia-owl', 'http://dbpedia.org/ontology/'],
	['dbpprop', 'http://dbpedia.org/property/'],
	['dbps', 'http://www.openlinksw.com/schemas/dbpedia-spotlight#'],
	['dc', 'http://purl.org/dc/elements/1.1/'],
	['dcat', 'http://www.w3.org/ns/dcat#'],
	['dcmitype', 'http://purl.org/dc/dcmitype/'],
	['dcterms', 'http://purl.org/dc/terms/'],
	['Demographics', 'http://www.zillow.com/static/xsd/Demographics.xsd'],
	['digg', 'http://digg.com/docs/diggrss/'],
	['doap', 'http://usefulinc.com/ns/doap#'],
	['dt', 'http://www.w3.org/2001/XMLSchema#'],
	['dv', 'http://rdf.data-vocabulary.org/'],
	['dwc', 'http://rs.tdwg.org/dwc/terms/'],
	['e', 'tag:extractiv.com,2010:rdf#'],
	['ebay', 'urn:ebay:apis:eBLBaseComponents'],
	['enc', 'http://purl.oclc.org/net/rss_2.0/enc#'],
	['ep', 'http://schemas.openxmlformats.org/officeDocument/2006/extended-properties'],
	['etsy', 'http://www.etsy.com/'],
	['event', 'http://purl.org/NET/c4dm/event.owl#'],
	['evri', 'http://www.openlinksw.com/schemas/evri#'],
	['exif', 'http://www.w3.org/2003/12/exif/ns/'],
	['fb', 'http://api.facebook.com/1.0/'],
	['fbase', 'http://rdf.freebase.com/ns/'],
	['fct', 'http://openlinksw.com/services/facets/1.0/'],
	['ff', 'http://api.friendfeed.com/2008/03'],
	['fn', 'http://www.w3.org/2005/xpath-functions/#'],
	['foaf', 'http://xmlns.com/foaf/0.1/'],
	['formats', 'http://www.w3.org/ns/formats/'],
	['g', 'http://base.google.com/ns/1.0'],
	['gb', 'http://www.openlinksw.com/schemas/google-base#'],
	['gbs', 'http://schemas.google.com/books/2008'],
	['gd', 'http://schemas.google.com/g/2005'],
	['geo', 'http://www.w3.org/2003/01/geo/wgs84_pos#'],
	['geonames', 'http://www.geonames.org/ontology#'],
	['georss', 'http://www.georss.org/georss'],
	['geospecies', 'http://rdf.geospecies.org/ont/geospecies#'],
	['getglue', 'http://getglue.com#'],
	['gh', 'http://www.openlinksw.com/schemas/github#'],
	['gm', 'http://base.google.com/ns-metadata/1.0'],
	['gml', 'http://www.opengis.net/gml'],
	['gn', 'http://www.geonames.org/ontology#'],
	['go', 'http://purl.org/obo/owl/GO#'],
	['gphoto', 'http://schemas.google.com/photos/2007'],
	['gr', 'http://purl.org/goodrelations/v1#'],
	['grs', 'http://www.georss.org/georss/'],
	['gs', 'http://schemas.google.com/spreadsheets/2006'],
	['guardian', 'http://www.openlinksw.com/schemas/guardian#'],
	['hlisting', 'http://www.openlinksw.com/schemas/hlisting/'],
	['hnews', 'http://ontologi.es/hnews#'],
	['hoovers', 'http://wwww.hoovers.com/'],
	['hrev', 'http:/www.purl.org/stuff/hrev#'],
	['http-voc', 'http://www.w3.org/2006/http#'],
	['ical', 'http://www.w3.org/2002/12/cal/ical#'],
	['ir', 'http://web-semantics.org/ns/image-regions'],
	['issues', 'http://schemas.google.com/projecthosting/issues/2009'],
	['itunes', 'http://www.itunes.com/DTDs/Podcast-1.0.dtd'],
	['lfm', 'http://last.fm/'],
	['lgv', 'http://linkedgeodata.org/vocabulary#'],
	['like', 'http://ontologi.es/like#'],
	['link', 'http://www.xbrl.org/2003/linkbase'],
	['lod', 'http://lod.openlinksw.com/'],
	['lu', 'http://lookupws.ontotext.com'],
	['m', 'http://schemas.microsoft.com/ado/2007/08/dataservices/metadata'],
	['math', 'http://www.w3.org/2000/10/swap/math#'],
	['md', 'http://www.w3.org/1999/xhtml/microdata#'],
	['media', 'http://search.yahoo.com/mrss/'],
	['mesh', 'http://purl.org/commons/record/mesh/'],
	['meta', 'urn:oasis:names:tc:opendocument:xmlns:meta:1.0'],
	['mf', 'http://www.w3.org/2001/sw/DataAccess/tests/test-manifest#'],
	['mmd', 'http://musicbrainz.org/ns/mmd-1.0#'],
	['mo', 'http://purl.org/ontology/mo/'],
	['moat', 'http://moat-project.org/ns#'],
	['mql', 'http://www.freebase.com/'],
	['nci', 'http://ncicb.nci.nih.gov/xml/owl/EVS/Thesaurus.owl#'],
	['ndfd_weather', 'http://www.openlinksw.com/schema/ndfd#'],
	['nfo', 'http://www.semanticdesktop.org/ontologies/nfo/#'],
	['ng', 'http://www.openlinksw.com/schemas/ning#'],
	['nyt', 'http://www.nytimes.com/'],
	['o', 'urn:schemas-microsoft-com:office:office'],
	['oai', 'http://www.openarchives.org/OAI/2.0/'],
	['oai_dc', 'http://www.openarchives.org/OAI/2.0/oai_dc/'],
	['obo', 'http://www.geneontology.org/formats/oboInOwl#'],
	['office', 'urn:oasis:names:tc:opendocument:xmlns:office:1.0'],
	['og', 'http://ogp.me/ns#'],
	['oo', 'urn:oasis:names:tc:opendocument:xmlns:meta:1.0:'],
	['opencyc', 'http://sw.opencyc.org/2008/06/10/concept/'],
	['openSearch', 'http://a9.com/-/spec/opensearchrss/1.0/'],
	['openstreetmap', 'http://openstreetmap.org/elements/'],
	['opl', 'http://www.openlinksw.com/schema/attribution#'],
	['oplamz', 'http://www.openlinksw.com/schemas/amazon#'],
	['oplangel', 'http://www.openlinksw.com/schemas/angel#'],
	['oplbase', 'http://www.openlinksw.com/schemas/oplbase#'],
	['oplbb', 'http://www.openlinksw.com/schemas/bestbuy#'],
	['oplbsbm', 'http://www.openlinksw.com/schemas/bsbm#'],
	['oplcars', 'http://www.openlinksw.com/schemas/cars#'],
	['oplcb', 'http://www.openlinksw.com/schemas/crunchbase#'],
	['oplcert', 'http://www.openlinksw.com/schemas/cert#'],
	['oplcn', 'http://www.openlinksw.com/schemas/cnet#'],
	['oplcv', 'http://www.openlinksw.com/schemas/cv#'],
	['oplebay', 'http://www.openlinksw.com/schemas/ebay#'],
	['oplevri', 'http://www.openlinksw.com/schemas/oplevri#'],
	['oplfactual', 'http://www.openlinksw.com/schemas/factual#'],
	['oplflattr', 'http://www.openlinksw.com/schemas/flattr#'],
	['oplfq', 'http://www.openlinksw.com/schemas/foursquare#'],
	['oplfr', 'http://www.openlinksw.com/schemas/federal_register#'],
	['oplgovt', 'http://www.openlinksw.com/schemas/govtrack#'],
	['oplgp', 'http://www.openlinksw.com/schemas/googleplus#'],
	['opl-gs', 'http://www.openlinksw.com/schemas/getsatisfaction/'],
	['oplgw', 'http://www.openlinksw.com/schemas/gowalla#'],
	['oplhp', 'http://www.openlinksw.com/schemas/hyperpublic#'],
	['oplie', 'http://www.openlinksw.com/schemas/influence#'],
	['opljg', 'http://www.openlinksw.com/schemas/jigsaw#'],
	['oplklout', 'http://www.openlinksw.com/schemas/klout#'],
	['oplli', 'http://www.openlinksw.com/schemas/linkedin#'],
	['opl-meetup', 'http://www.openlinksw.com/schemas/meetup/'],
	['oplmisc', 'http://www.openlinksw.com/schemas/oplmisc#'],
	['oplmny', 'http://www.openlinksw.com/schemas/money#'],
	['oplnyt', 'http://www.openlinksw.com/schemas/nyt#'],
	['oplog', 'http://www.openlinksw.com/schemas/opengraph#'],
	['oplomim', 'http://www.openlinksw.com/schemas/omim#'],
	['oplpingar', 'http://www.openlinksw.com/schemas/pingar#'],
	['oplrdfs', 'http://www.openlinksw.com/schemas/rdfs/'],
	['oplsc', 'http://www.openlinksw.com/schemas/soundcloud#'],
	['oplseevl', 'http://www.openlinksw.com/schemas/seevl#'],
	['oplsg', 'http://www.openlinksw.com/schemas/simplegeo#'],
	['oplshodan', 'http://www.openlinksw.com/schemas/shodan#'],
	['oplso', 'http://www.openlinksw.com/schemas/stackoverflow#'],
	['opltable', 'http://www.openlinksw.com/schemas/opltable#'],
	['opltesco', 'http://www.openlinksw.com/schemas/tesco#'],
	['opltw', 'http://www.openlinksw.com/schemas/twitter#'],
	['oplustream', 'http://www.openlinksw.com/schemas/ustream#'],
	['oplweb', 'http://www.openlinksw.com/schemas/oplweb#'],
	['oplwthr', 'http://www.openlinksw.com/schemas/weatherreport#'],
	['oplwtp', 'http://www.openlinksw.com/schemas/wtp#'],
	['opl-xbrl', 'http://www.openlinksw.com/schemas/xbrl/'],
	['oplxg', 'http://www.openlinksw.com/schemas/xing#'],
	['oplyls', 'http://www.openlinksw.com/schemas/yahoo_local_search#'],
	['oplzllw', 'http://www.openlinksw.com/schemas/zillow#'],
	['ore', 'http://www.openarchives.org/ore/terms/'],
	['owl', 'http://www.w3.org/2002/07/owl#'],
	['p', 'http://schemas.openxmlformats.org/presentationml/2006/main'],
	['pdo', 'http://ontologies.smile.deri.ie/pdo#'],
	['ping', 'http://prodapi.pingar.com/'],
	['pingns', 'http://prodapi.pingar.com'],
	['plancast', 'http://www.openlinksw.com/schema/plancast#'],
	['po', 'http://purl.org/ontology/po/'],
	['poco', 'http://portablecontacts.net/ns/1.0'],
	['pres', 'http://virtuoso.openlinksw.com/presentations/'],
	['primal', 'http://api.primal.com/primal#'],
	['product', 'http://www.buy.com/rss/module/productV2/'],
	['protseq', 'http://purl.org/science/protein/bysequence/'],
	['proxy', 'http://www.openlinksw.com/about/id/'],
	['prv', 'http://purl.org/net/provenance/ns#'],
	['prvTypes', 'http://purl.org/net/provenance/types#'],
	['pto', 'http://www.productontology.org/id/'],
	['pw', 'http://www.programmableweb.com/api/opensearch/1.0/'],
	['r', 'http://backend.userland.com/rss2'],
	['radio', 'http://www.radiopop.co.uk/'],
	['rdf', 'http://www.w3.org/1999/02/22-rdf-syntax-ns#'],
	['rdfa', 'http://www.w3.org/ns/rdfa#'],
	['rdfdf', 'http://www.openlinksw.com/virtrdf-data-formats#'],
	['rdfg', 'http://www.w3.org/2004/03/trix/rdfg-1/'],
	['rdfs', 'http://www.w3.org/2000/01/rdf-schema#'],
	['realdf', 'http://gr8c.org/realdf/ns#'],
	['redwood-tags', 'http://www.holygoat.co.uk/owl/redwood/0.1/tags/'],
	['rev', 'http://purl.org/stuff/rev#'],
	['review', 'http:/www.purl.org/stuff/rev#'],
	['rnews', 'http://iptc.org/std/rnews/20110202/rnews.owl#'],
	['rsa', 'http://www.w3.org/ns/auth/rsa#'],
	['rss', 'http://purl.org/rss/1.0/'],
	['s', 'http://www.google.com/shopping/api/schemas/2010'],
	['sc', 'http://purl.org/science/owl/sciencecommons/'],
	['schema', 'http://schema.org/'],
	['schemavideo', 'http://schema.org/VideoObject#'],
	['scot', 'http://scot-project.org/scot/ns#'],
	['scovo', 'http://purl.org/NET/scovo#'],
	['sd', 'http://www.w3.org/ns/sparql-service-description#'],
	['SearchResults', 'http://www.zillow.com/static/xsd/SearchResults.xsd'],
	['sf', 'urn:sobject.enterprise.soap.sforce.com'],
	['sg', 'http://www.seatgeek.com#'],
	['sioc', 'http://rdfs.org/sioc/ns#'],
	['sioct', 'http://rdfs.org/sioc/types#'],
	['sitemap', 'http://www.sitemaps.org/schemas/sitemap/0.9'],
	['skos', 'http://www.w3.org/2004/02/skos/core#'],
	['slash', 'http://purl.org/rss/1.0/modules/slash/'],
	['spo', 'http://www.openlinksw.com/schemas/sponger/'],
	['sql', 'sql:'],
	['ss', 'urn:schemas-microsoft-com:office:spreadsheet'],
	['stock', 'http://xbrlontology.com/ontology/finance/stock_market#'],
	['taxconcept', 'http://www.taxonconcept.org/ont/se_v01/seo.owl#'],
	['tesco', 'http://www.tesco.com/'],
	['thr', 'http://purl.org/syndication/thread/1.0'],
	['time', 'http://www.w3.org/2006/time#'],
	['tio', 'http://purl.org/tio/ns#'],
	['tk', 'http://www.trueknowledge.com/ns/kengine'],
	['tl', 'http://purl.org/NET/c4dm/timeline.owl#'],
	['twfy', 'http://www.openlinksw.com/schemas/twfy#'],
	['twitter', 'http://www.openlinksw.com/schemas/twitter/'],
	['uc', 'http://api.uclassify.com/1/ResponseSchema#'],
	['umbel', 'http://umbel.org/umbel#'],
	['umbel-ac', 'http://umbel.org/umbel/ac/'],
	['umbel-sc', 'http://umbel.org/umbel/sc/'],
	['uniprot', 'http://purl.uniprot.org/'],
	['UpdatedPropertyDetails', 'http://www.zillow.com/static/xsd/UpdatedPropertyDetails.xsd'],
	['usc', 'http://www.rdfabout.com/rdf/schema/uscensus/details/100pct/'],
	['v', 'http://www.openlinksw.com/xsltext/'],
	['vcard', 'http://www.w3.org/2001/vcard-rdf/3.0#'],
	['vcard2006', 'http://www.w3.org/2006/vcard/ns#'],
	['vi', 'http://www.openlinksw.com/virtuoso/xslt/'],
	['video', 'http://purl.org/media/video#'],
	['vimeo', 'http://vimeo.com/'],
	['virt', 'http://www.openlinksw.com/virtuoso/xslt'],
	['virtcxml', 'http://www.openlinksw.com/schemas/virtcxml#'],
	['virtrdf', 'http://www.openlinksw.com/schemas/virtrdf#'],
	['virtrdfmec', 'http://www.openlinksw.com/schemas/virtrdf-meta-entity-class#'],
	['void', 'http://rdfs.org/ns/void#'],
	['vso', 'http://purl.org/vso/ns#'],
	['vt', 'http://schemas.openxmlformats.org/officeDocument/2006/docPropsVTypes'],
	['w', 'http://schemas.microsoft.com/office/word/2003/wordml'],
	['wa', 'http://www.openlinksw.com/schemas/wolframalpha#'],
	['wb', 'http://www.worldbank.org/'],
	['wdrs', 'http://www.w3.org/2007/05/powder-s#'],
	['web', 'http://schemas.microsoft.com/LiveSearch/2008/04/XML/web'],
	['wf', 'http://www.w3.org/2005/01/wf/flow#'],
	['wfw', 'http://wellformedweb.org/CommentAPI/'],
	['xf', 'http://www.w3.org/2004/07/xpath-functions'],
	['xfn', 'http://gmpg.org/xfn/11#'],
	['xfnv', 'http://vocab.sindice.com/xfn#'],
	['xhtml', 'http://www.w3.org/1999/xhtml'],
	['xhv', 'http://www.w3.org/1999/xhtml/vocab#'],
	['xi', 'http://www.xbrl.org/2003/instance'],
	['xlink', 'http://www.w3.org/1999/xlink'],
	['xml', 'http://www.w3.org/XML/1998/namespace'],
	['xn', 'http://www.ning.com/atom/1.0'],
	['xsd', 'http://www.w3.org/2001/XMLSchema#'],
	['xsi', 'http://www.w3.org/2001/XMLSchema-instance'],
	['xsl', 'http://www.w3.org/1999/XSL/Transform'],
	['xsl10', 'http://www.w3.org/XSL/Transform/1.0'],
	['xsl1999', 'http://www.w3.org/1999/XSL/Transform'],
	['xslwd', 'http://www.w3.org/TR/WD-xsl'],
	['y', 'urn:yahoo:maps'],
	['yago', 'http://dbpedia.org/class/yago/'],
	['yahoo_weather', 'http://www.openlinksw.com/schema/yweather#'],
	['yca', 'urn:yahoo:cap'],
	['yql', 'http://www.openlinksw.com/schemas/yql#'],
	['yt', 'http://gdata.youtube.com/schemas/2007'],
	['yweather', 'http://xml.weather.yahoo.com/ns/rss/1.0'],
	['zem', 'http://s.zemanta.com/ns#'],
	['zillow', 'http://www.zillow.com/']
];