SemwidgJS.EventDispatcher = function() {

};

SemwidgJS.EventDispatcher.prototype.__getUID = function() {
	if (!this.__uid) {
		this.__uid = SemwidgJS.Util.getUniqueId();
	}

	return this.__uid;
};

SemwidgJS.EventDispatcher.prototype._addEventListener = function(type, callback, owner) {
	if (!this._getCallbacks()[type]) {
		// this._getCallbacks()[type] = $.Callbacks();
		this._getCallbacks()[type] = new SemwidgJS.Callbacks(owner);
	}
	this._getCallbacks()[type].add(callback);

	SemwidgJS.EventDispatcher._addToEventListenerMap(owner.__getUID(), 
		new SemwidgJS.EventDispatcher.EventListenerMapEntry(owner, type, callback, this)
	);
};

SemwidgJS.EventDispatcher.prototype._removeEventListener = function(type, callback) {
	if (!this._getCallbacks()[type]) {
		return;
	}
	this._getCallbacks()[type].remove(callback);
};

SemwidgJS.EventDispatcher.prototype._dispatchEvent = function(event) {

	if (event && event instanceof SemwidgJS.Event && event.getType()) {
		
		if (this._getCallbacks()[event.getType()]) {

			event.setDispatcher(this);
			this._getCallbacks()[event.getType()].fire(event);

		}
		
	}else {
		console.error('Dispatched object is not an event. It is: ' + SemwidgJS.Util.getClass(event));
	}
};

SemwidgJS.EventDispatcher.prototype._getCallbacks = function() {
	if (!this._callbacks) {
		this._callbacks = {};
	}

	return this._callbacks;
}

SemwidgJS.Callbacks = function(owner) {
	this._owner = owner;
	
};

SemwidgJS.Callbacks.prototype.add = function(callback) {
	if (typeof callback == 'function') {
		this.getCallbackList().push(callback);
	}
};

SemwidgJS.Callbacks.prototype.remove = function(callback) {
	var indexes = [];

	for (var i = 0; i < this.getCallbackList().length; i++) {
		if (this.getCallbackList()[i] == callback) {
			indexes.push(i);
		}
	}
	if (indexes.length > 0) {
		for (var j = indexes.length - 1; j >= 0; j--) {
			this.getCallbackList().splice(indexes[j], 1);
		}
	}
};

SemwidgJS.Callbacks.prototype.fire = function(event) {
	for (var i = 0; i < this.getCallbackList().length; i++) {
		// console.log('Calling', this.getCallbackList()[i], this._owner, event);
		this.getCallbackList()[i].call(this._owner, event);
	}
};

SemwidgJS.Callbacks.prototype.getCallbackList = function() {
	if(!this._callbackList) {
		this._callbackList = [];
	}
	return this._callbackList;
};

SemwidgJS.EventDispatcher.eventListenerMap = {};

SemwidgJS.EventDispatcher.EventListenerMapEntry = function(owner, type, callbackFunction, eventDispatcherObj) {
	this.owner = owner;
	this.type = type;
	this.callbackFunction = callbackFunction;
	this.eventDispatcherObj = eventDispatcherObj;
};

SemwidgJS.EventDispatcher._addToEventListenerMap = function(ownerId, eventListenerMapEntry) {
	if (!SemwidgJS.EventDispatcher.eventListenerMap[ownerId]) {
		SemwidgJS.EventDispatcher.eventListenerMap[ownerId] = [];
	}

	SemwidgJS.EventDispatcher.eventListenerMap[ownerId].push(eventListenerMapEntry);
};

SemwidgJS.EventDispatcher._getEventListenerMapEntriesByOwner = function(owner) {
	var ownerId = owner.__getUID();

	if (!SemwidgJS.EventDispatcher.eventListenerMap[ownerId]) {
		return [];
	}

	return SemwidgJS.EventDispatcher.eventListenerMap[ownerId];
};

SemwidgJS.EventDispatcher._getEventListenerMapEntriesByEventDispatcher = function(eventDispatcher) {
	var entries = [];
	var e, e2;

	for (var entryUID in SemwidgJS.EventDispatcher.eventListenerMap) {
		e = SemwidgJS.EventDispatcher.eventListenerMap[entryUID];
		for (var i = 0; i < e.length; i++) {
			e2 = e[i];
			if (e2.eventDispatcherObj == eventDispatcher) {
				entries.push(e2);
			}
		}
	}
	return entries;
};

SemwidgJS.EventDispatcher._removeEventListenerMapEntry = function(entry) {
	
	var e;
	for (var entryUID in SemwidgJS.EventDispatcher.eventListenerMap) {
		e = SemwidgJS.EventDispatcher.eventListenerMap[entryUID];
		for (var i = e.length - 1; i >= 0; i--) {
			if (e[i] == entry) {
				e[i] = null;
				e.splice(i, 1);
			}
		}

		if (e.length == 0) {
			delete SemwidgJS.EventDispatcher.eventListenerMap[entryUID];
		}
	}
};

SemwidgJS.EventDispatcher._removeAllReferences = function(element) {

	var entry;

	// remove event listeners to this element
	var entries2 = SemwidgJS.EventDispatcher._getEventListenerMapEntriesByEventDispatcher(element);
	for (var i = 0; i < entries2.length; i++) {
		entry = entries2[i];
		entry.eventDispatcherObj._removeEventListener(entry.type, entry.callbackFunction);
		SemwidgJS.EventDispatcher._removeEventListenerMapEntry(entry);
	}

	// remove own event listeners from this element
	var entries = SemwidgJS.EventDispatcher._getEventListenerMapEntriesByOwner(element);
	for (var i = 0; i < entries.length; i++) {
		entry = entries[i];
		entry.owner._removeEventListener(entry.type, entry.callbackFunction);
	}

	element._callbacks = null;
};