SemwidgJS.HtmlElement = function() {

};

SemwidgJS.HtmlElement.prototype = new SemwidgJS.EventDispatcher();

SemwidgJS.HtmlElement.prototype.getId = function() {
	return this.id;
};

SemwidgJS.HtmlElement.prototype.setId = function(id, override) {
	// An id must not be changed

	if ((this.id === undefined || override == true) && !SemwidgJS.Util.String.isEmpty(id)) {
		this.id = id;
		this._dispatchEvent(new SemwidgJS.Event('idChanged', id));
	}else {

		if (SemwidgJS.Util.String.isEmpty(id)) {
			console.warn('Id is empty or null. This is not allowed.');
		}else if (id != this.id) {
			console.warn('Tried to change id of ' + this.id + '. This is not allowed.');
		}
	}
};

SemwidgJS.HtmlElement.prototype._generateId = function(prefix) {

	if (!SemwidgJS.Util.String.isEmpty(prefix)) {
		prefix = prefix + ':';
	}else {
		prefix = '';
	}

	var id = prefix
			+ 'Semwidg_internal_ID_' + this._getType()
			+ '_' + SemwidgJS.Util.getIdCounter() 
			+ '_' + SemwidgJS.getInstanceId();

	this.setId(id);
};

SemwidgJS.HtmlElement.prototype._getType = function() {
	return 'SwHtmlElement';
};

SemwidgJS.HtmlElement.prototype._getNode = function() {
	return this._node;
};

SemwidgJS.HtmlElement.prototype._setNode = function(node) {
	
	// disconnect Mutation Observer form previous node
	if (this._node) {
		this._getMutationObserver().disconnect();
		this._observer = null;
	}

	this._node = node;

	// observe attribute changes
	if (this._node) {
		var options = {attributes: true};
		this._getMutationObserver().observe(this._node, options);
	}
};

SemwidgJS.HtmlElement.prototype._getMutationObserver = function() {
	
	if (!this._observer) {

		var owner = this;

		this._observer = new MutationObserver(function (mutations) {
			mutations.forEach(function (mutation) {

				var thisOwner = owner;
				
				if (mutation.attributeName && SemwidgJS.Util.String.startsWith(
						mutation.attributeName, 'data-swp-')) {

					var attribute = mutation.attributeName.slice(5);

					//clear jQuery's data cache for this attribute
					$(mutation.target).removeData(attribute);
					
					var value = SemwidgJS.Util.String.htmlUnescape($(mutation.target).data(attribute));
					var propertyName = SemwidgJS.Util.String.toUpperCamelCase(mutation.attributeName.slice(9));
					var functionName = 'set' + propertyName;

					if (typeof thisOwner[functionName] === 'function') {
						thisOwner[functionName].call(thisOwner, value);
					}else {
						console.warn('Property "' + propertyName + '" is not available for ' + this.owner.getId());
					}
				}
			})
		});
	}

	return this._observer;
};

SemwidgJS.HtmlElement.prototype._writeToDom = function(propertyName) {

	var attributeName = 'data-swp-' + propertyName;

	if ($(this._getNode()).attr(attributeName) && this[propertyName]
		&& $(this._getNode()).attr(attributeName) !== this[propertyName]) {
		$(this._getNode()).attr(attributeName, this[propertyName]);
	}

};
