SemwidgJS.ParameterObject.Part = function() {
	this.id = SemwidgJS.Util.getUniqueId();
	this.isLoading = false;
};

SemwidgJS.ParameterObject.Part.prototype = new SemwidgJS.EventDispatcher();

SemwidgJS.ParameterObject.Part.prototype.getId = function() {
	return this.id;
};

SemwidgJS.ParameterObject.Part.prototype.getParameterObject = function() {
	return this.parameterObject;
};

SemwidgJS.ParameterObject.Part.prototype.setParameterObject = function(parameterObject) {
	this.parameterObject = parameterObject;
};

SemwidgJS.ParameterObject.Part.prototype.getType = function() {
	return this.type;
};

SemwidgJS.ParameterObject.Part.prototype.setType = function(type) {
	this.type = type;
	this._dispatchEvent(new SemwidgJS.Event('typeChanged', type));
};

SemwidgJS.ParameterObject.Part.prototype.getOriginalString = function() {
	return this.originalString;
};

SemwidgJS.ParameterObject.Part.prototype.setOriginalString = function(originalString) {
	this.originalString = originalString;
	this._dispatchEvent(new SemwidgJS.Event('originalStringChanged', originalString));
};

SemwidgJS.ParameterObject.Part.prototype.getQueryString = function() {
	return this.queryString;
};

SemwidgJS.ParameterObject.Part.prototype.setQueryString = function(queryString) {
	if (this.queryString !== queryString) {
		if (queryString == 'null') {
			queryString = null;
		}
		this.queryString = queryString;
		this._dispatchEvent(new SemwidgJS.Event('queryStringChanged', queryString));

		this.setSparqlResult(null);
		// request Data
		SemwidgJS.SparqlConnection.requestDataForParameterObjectPart(this);
	}
};

SemwidgJS.ParameterObject.Part.prototype.getSparqlResult = function() {
	return this.results;
};

SemwidgJS.ParameterObject.Part.prototype.setSparqlResult = function(results, index) {
	if (!this.results) {
		this.results = [];
	}
	if (!index) {
		index = 0;
	}
	this.results[index] = results;
	this._dispatchEvent(new SemwidgJS.Event('resultsChanged', results));
};

SemwidgJS.ParameterObject.Part.prototype.getNamedResources = function() {
	return this.namedResources;
};

SemwidgJS.ParameterObject.Part.prototype.setNamedResources = function(namedResources) {
	this.namedResources = namedResources;
};

SemwidgJS.ParameterObject.Part.prototype.getMappings = function() {
	return this.mappings;
};

SemwidgJS.ParameterObject.Part.prototype.setMappings = function(mappings) {
	this.mappings = mappings;
};

SemwidgJS.ParameterObject.Part.prototype.getMappingDelimiter = function() {
	return '$$';
};

SemwidgJS.ParameterObject.Part.prototype.process = function() {

	this.findMappings();
	this.findNamedResources();
	this.updateQueryString();

};

SemwidgJS.ParameterObject.Part.prototype.findMappings = function() {

	var str = this.getOriginalString();

	// TODO: remove event listener
	this.setMappings(null);

	var mappingDelimiter = this.getMappingDelimiter();
	var mappingDelimiterIndeces = SemwidgJS.Util.String.getIndicesOf(str, mappingDelimiter, false);
	if (mappingDelimiterIndeces && mappingDelimiterIndeces.length >= 2) {
		
		if (mappingDelimiterIndeces.length % 2 != 0) {
			mappingDelimiterIndeces.pop();
		}

		var mappingNames = [];
		for (var i = 0; i < mappingDelimiterIndeces.length; i = i + 2) {
			mappingNames.push(str.substring(mappingDelimiterIndeces[i] + mappingDelimiter.length, mappingDelimiterIndeces[i + 1]));
		}

		var mappings = [];
		for (var i = 0; i < mappingNames.length; i++) {
			var mapping = SemwidgJS.getConfig().getMapping(mappingNames[i]);
			if (mapping) {
				mappings.push(mapping);
			}else {
				console.warn('Mapping ' + mappingNames[i] + ' was not found. "' 
						+ this.getParameterObject().getWidget().getId() 
						+ '" will probably not work.');
			}
		}
		
		this.setMappings(mappings);
	}

	this.addMappingEventListener();
};

SemwidgJS.ParameterObject.Part.prototype.addMappingEventListener = function() {

	var owner = this;

	var handleMappingChanged = function(event) {
		this.owner = owner;
		this.owner.updateQueryString();
	};

	if (this.getMappings()) {
		for (var i = 0; i < this.getMappings().length; i++) {
			var mapping = this.getMappings()[i];
			mapping._addEventListener('mappingChanged', handleMappingChanged, this);
		}
	}

};

SemwidgJS.ParameterObject.Part.prototype.findNamedResources = function() {


	// TODO: remove event listener
	this.setNamedResources(null);

	var type = this._parseType();

	if (type == 'semwidgql') {

		var str = this.getOriginalStringWithMapping();
		var query = str.substring(1, str.length - 1);

		var resources = SemwidgQL.getNamedResources(query);

		var namedResources = [];
		for (var i = 0; i < resources.length; i++) {
			var resource = SemwidgJS.getConfig().getResource(resources[i]);
			if (resource) {
				namedResources.push(resource);
			}
		}
		
		this.setNamedResources(namedResources);
		this.addNamedResourceEventListener();
	}
};

SemwidgJS.ParameterObject.Part.prototype.addNamedResourceEventListener = function() {

	var owner = this;

	var handleNamedResourceChanged = function(event) {
		this.owner = owner;
		this.owner.updateQueryString();
	};

	if (this.getNamedResources()) {
		for (var i = 0; i < this.getNamedResources().length; i++) {
			var resource = this.getNamedResources()[i];
			resource._addEventListener('resourceChanged', handleNamedResourceChanged, this);
		}
	}

};

SemwidgJS.ParameterObject.Part.prototype.updateQueryString = function() {

	if (!this.getParameterObject()) {
		return;
	}
	
	var str = this.getOriginalStringWithMapping();

	this.setType(this._parseType());

	var name = this.getParameterObject().getParameterName();
	var limit = this.getParameterObject().getWidget()['get' + name + 'Limit']();

	if (SemwidgJS.Util.String.isEmpty(limit)) {
		limit = this.getParameterObject().getWidget().getLimit();
	}

	if (SemwidgJS.Util.String.isEmpty(limit)) {
		limit = null;
	}
	
	if (this.getType() === 'semwidgql') {

		this.isLoading = true;

		var query = str.substring(1, str.length - 1);
		this.setQueryString(SemwidgJS.Util.SemwidgQL.toSparql(query, limit));

	}else if (this.getType() == 'sparql') {

		this.isLoading = true;

		var query = str.substring(2, str.length - 2);
		this.setQueryString([query]);

	}else {

		this.isLoading = false;
		this.setQueryString(str);
	}

};

SemwidgJS.ParameterObject.Part.prototype._parseType = function() {

	var str = this.getOriginalStringWithMapping();

	if (SemwidgJS.Util.String.isEmpty(str)) {
		return 'string';
	}

	var length = str.length;

	// simple text
	if (str.charAt(0) != '{') {
		return 'string';

	// SemwidgQL or SPARQL query
	}else {

		if (length > 1) {

			// SemwidgQL query, start with '{'
			if (str.charAt(1) != '{') {

				if (length >= 2 && str.charAt(length - 1) == '}') {

					var query = str.substring(1, length - 1);

					// empty SemwidgQL query
					if (SemwidgJS.Util.String.isEmpty(query)) {
						console.warn('Empty SemwidgQL query');
						return 'string';

					// valid query
					}else {
						return 'semwidgql';
					}

				// malformed 
				}else {
					console.error('Malformed parameter. Missing "}"?\n' + str);
					return 'string';
				}

			// SPARQL query, starts with '{{'
			}else {

				// ends with '}}'
				if (length >= 4 && str.charAt(length - 2) == '}' && str.charAt(length - 1) == '}') {

					var query = str.substring(2, length - 2);

					// empty SPARQL query
					if (SemwidgJS.Util.String.isEmpty(query)) {
						console.warn('Empty SPARQL query');
						return 'string';

					// valid query
					}else {
						return 'sparql';
					}

				// malformed 
				}else {
					console.error('Malformed parameter. Missing "}"?\n' + str);
					return 'string';
				}
			}

		// single character '{'
		}else {
			// return 'string';
		}
	}

	return 'string';

};

SemwidgJS.ParameterObject.Part.prototype.getOriginalStringWithMapping = function() {
	var str = this.getOriginalString();

	var mappingDelimiter = this.getMappingDelimiter();
	if (this.getMappings()) {
		for (var i = 0; i < this.getMappings().length; i++) {
			var mapping = this.getMappings()[i];
			
			var mappingStr = mappingDelimiter + mapping.getId() + mappingDelimiter;
			var mappingValue = mapping.getValue();
			str = str.replace(mappingStr, mappingValue);
		}
	}

	return str;
};

// SemwidgJS.ParameterObject.Part.prototype.isLoading = false;
