SemwidgJS.SparqlCache = function() {
	
};

// SemwidgJS.SparqlCache.getCache().clear();

SemwidgJS.SparqlCache.getCache = function() {

	if (!SemwidgJS.SparqlCache.__cache) {
		if(typeof(window.localStorage) !== 'undefined' && !SemwidgJS.internalCacheOnly) {
			console.log('Using Local Storage');
			SemwidgJS.SparqlCache.__cache = window.localStorage;

			console.log('Removing old ResultObjects')
			SemwidgJS.SparqlCache._removeOldestResultObjects(7);

		} else {
			console.log('Using Internal Storage');
			SemwidgJS.SparqlCache.__cache = new SemwidgJS.SparqlCache.Storage();
		}
	}

	return SemwidgJS.SparqlCache.__cache;

};

SemwidgJS.SparqlCache.getResult = function(sparqlQuery, endpointUrl, defaultGraphUri, format) {

	var obj = new SemwidgJS.SparqlCache.ResultObject();
	obj.setSparqlQuery(sparqlQuery);
	obj.setEndpointUrl(endpointUrl);
	obj.setDefaultGraphUri(defaultGraphUri);
	obj.setFormat(format);
	
	var objHash = obj.getConfigHash();
	
	var result = SemwidgJS.SparqlCache.deserialize(
			SemwidgJS.SparqlCache.getCache().getItem(objHash));


	// not found or searching for more than 2 minutes
	if (result && result.getStatus() != 1) {
		var now = new Date();
		if (now.getTime() - result.getLastUpate().getTime() > 120000) {
			return null;
		}
	}

	return result;
	
};

SemwidgJS.SparqlCache.insertResult = function(sparqlQuery, endpointUrl, defaultGraphUri, format, status, result) {

	var obj = new SemwidgJS.SparqlCache.ResultObject();
	obj.setSparqlQuery(sparqlQuery);
	obj.setEndpointUrl(endpointUrl);
	obj.setDefaultGraphUri(defaultGraphUri);
	obj.setFormat(format);
	obj.setStatus(status);
	obj.setResult(result);
	obj.setLastUpate(new Date());

	var objHash = obj.getConfigHash();

	var serialized = SemwidgJS.SparqlCache.serialize(obj);

	try {
		SemwidgJS.SparqlCache.getCache().setItem(objHash, serialized);
	}catch(e) {

		if (SemwidgJS.showExceptions) {
			console.error(e);
		}

		if (SemwidgJS.SparqlCache.getCache().length > 0) {
			SemwidgJS.SparqlCache._removeOldestResultObject();
			return SemwidgJS.SparqlCache.insertResult(sparqlQuery, endpointUrl, defaultGraphUri, format, status, result);
		}
	}

	return obj;
	
};

SemwidgJS.SparqlCache._removeOldestResultObjects = function(beforeDays) {
	var today = new Date();
	var previousDate = new Date(new Date().setDate(today.getDate() - beforeDays));

	if (SemwidgJS.SparqlCache.__cache) {
		var key;
		var item;
		var object;

		var count = 0;

		for (var i = 0; i < SemwidgJS.SparqlCache.__cache.length; i++) {
			key = SemwidgJS.SparqlCache.__cache.key(i);
			item = SemwidgJS.SparqlCache.__cache.getItem(key);
			object = SemwidgJS.SparqlCache.deserialize(item);

			if (!object || !object.getLastUpate() || object.getLastUpate() < previousDate) {
				SemwidgJS.SparqlCache.__cache.removeItem(key);
				count++;
			}
		}

		console.log('Removed ' + count + ' outdated ResultObject(s) older than ' + beforeDays + ' days.');
		
	}
};

SemwidgJS.SparqlCache._removeOldestResultObject = function() {
	if (SemwidgJS.SparqlCache.__cache) {
		var key;
		var item;
		var object;

		var oldestKey;
		var oldestDate = new Date();

		for (var i = 0; i < SemwidgJS.SparqlCache.__cache.length; i++) {
			key = SemwidgJS.SparqlCache.__cache.key(i);
			item = SemwidgJS.SparqlCache.__cache.getItem(key);
			object = SemwidgJS.SparqlCache.deserialize(item);
			
			if (!object.getLastUpate() || object.getLastUpate() < oldestDate) {
				oldestDate = object.getLastUpate();
				oldestKey = key;
			}
		}

		if (oldestKey) {
			SemwidgJS.SparqlCache.__cache.removeItem(key);
		}

	}
};

SemwidgJS.SparqlCache.Storage = function() {

};

SemwidgJS.SparqlCache.Storage.prototype.setItem = function(key, value) {
	this.getStorage()[key] = value;
};

SemwidgJS.SparqlCache.Storage.prototype.getItem = function(key) {
	return this.getStorage()[key];
};

SemwidgJS.SparqlCache.Storage.prototype.removeItem = function(key) {
	delete this.getStorage()[key];
};

SemwidgJS.SparqlCache.Storage.prototype.clear = function() {
	this._storage = new Object();
};

SemwidgJS.SparqlCache.Storage.prototype.key = function(index) {
	//TODO
	console.log(this._storage);
};

SemwidgJS.SparqlCache.Storage.prototype.length = function() {
	//TODO
};

SemwidgJS.SparqlCache.Storage.prototype.getStorage = function() {
	if (!this._storage) {
		this._storage = new Object();
	}

	return this._storage;
};

SemwidgJS.SparqlCache.serialize = function(resultObject) {

	if (resultObject instanceof SemwidgJS.SparqlCache.ResultObject) {
		var serialized = JSON.stringify(resultObject);
		return serialized;
	}
	
	return null;
};

SemwidgJS.SparqlCache.deserialize = function(serialized) {
	
	if (!serialized) {
		return null;
	}

	var obj;

	try {
		obj = JSON.parse(serialized);
	}catch (e) {

		if (SemwidgJS.showExceptions) {
			console.error(e);
		}
		
		return null;
	}

	if (!obj) {
		return null;
	}

	var resultObject = new SemwidgJS.SparqlCache.ResultObject();
	resultObject.setSparqlQuery(obj['sparqlQuery']);
	resultObject.setEndpointUrl(obj['endpointUrl']);
	resultObject.setDefaultGraphUri(obj['defaultGraphUri']);
	resultObject.setFormat(obj['format']);
	resultObject.setStatus(obj['status']);
	resultObject.setResult(obj['result']);
	resultObject.setLastUpate(new Date(obj['lastUpdate']));

	return resultObject;

};

SemwidgJS.SparqlCache.ResultObject = function() {

};

SemwidgJS.SparqlCache.ResultObject.STATUS_NOT_FOUND = 0;

SemwidgJS.SparqlCache.ResultObject.STATUS_FOUND = 1;

SemwidgJS.SparqlCache.ResultObject.STATUS_REQUESTING = 2;

SemwidgJS.SparqlCache.ResultObject.prototype.getStatus = function() {
	return this.status;
};

SemwidgJS.SparqlCache.ResultObject.prototype.setStatus = function(status) {
	this.status = status;
};

SemwidgJS.SparqlCache.ResultObject.prototype.getLastUpate = function() {
	return this.lastUpdate;
};

SemwidgJS.SparqlCache.ResultObject.prototype.setLastUpate = function(lastUpdate) {
	this.lastUpdate = lastUpdate;
};

SemwidgJS.SparqlCache.ResultObject.prototype.getSparqlQuery = function() {
	if (!this.sparqlQuery) {
		return '';
	}
	return this.sparqlQuery;
};

SemwidgJS.SparqlCache.ResultObject.prototype.setSparqlQuery = function(sparqlQuery) {
	this.sparqlQuery = sparqlQuery;
};

SemwidgJS.SparqlCache.ResultObject.prototype.getEndpointUrl = function() {
	if (!this.endpointUrl) {
		return '';
	}
	return this.endpointUrl;
};

SemwidgJS.SparqlCache.ResultObject.prototype.setEndpointUrl = function(endpointUrl) {
	this.endpointUrl = endpointUrl;
};

SemwidgJS.SparqlCache.ResultObject.prototype.getDefaultGraphUri = function() {
	if (!this.defaultGraphUri) {
		return '';
	}
	return this.defaultGraphUri;
};

SemwidgJS.SparqlCache.ResultObject.prototype.setDefaultGraphUri = function(defaultGraphUri) {
	this.defaultGraphUri = defaultGraphUri;
};

SemwidgJS.SparqlCache.ResultObject.prototype.getFormat = function() {
	if (!this.format) {
		return '';
	}
	return this.format;
};

SemwidgJS.SparqlCache.ResultObject.prototype.setFormat = function(format) {
	this.format = format;
};

SemwidgJS.SparqlCache.ResultObject.prototype.getResult = function() {
	if (!this.result) {
		return '';
	}
	return this.result;
};

SemwidgJS.SparqlCache.ResultObject.prototype.setResult = function(result) {
	this.result = result;
};

SemwidgJS.SparqlCache.ResultObject.prototype.getConfigHash = function() {

	var str = this.getSparqlQuery() + this.getEndpointUrl() + this.getDefaultGraphUri() + this.getFormat();

	var hash = 0;
	var strlen = str.length;
	var i;
	var c;

	if (strlen === 0) {
		return hash;
	}

	for (i = 0; i < strlen; i++) {
		c = str.charCodeAt(i);
		hash = ((hash << 5) - hash) + c;
		hash = hash & hash; // Convert to 32bit integer
	}

	return 'X' + hash;

};

SemwidgJS.SparqlCache.ResultObject.prototype.equalsConfiguration = function(other) {

	return (this.getConfigHash() == other.getConfigHash());

};