SemwidgJS.SparqlConnection = function() {

};

SemwidgJS.SparqlConnection.requestData = function(widget, property) {

	var parameterObject = widget._getParameterObjects()[property];

	if (!parameterObject) {
		return;
	}

	var parts = parameterObject.getParameterObjectParts();

	for (var i = 0; i < parts.length; i++) {

		SemwidgJS.SparqlConnection.requestDataForParameterObjectPart(parts[i]);
	}
};

SemwidgJS.SparqlConnection.timeouts = {};
SemwidgJS.SparqlConnection.loadingSince = {};
SemwidgJS.SparqlConnection.requestDataForParameterObjectPart = function(parameterObjectPart) {
	if (!parameterObjectPart || !parameterObjectPart.getParameterObject()) {
		return;
	}

	var part = parameterObjectPart;
	var widget = parameterObjectPart.getParameterObject().getWidget();
	var property = parameterObjectPart.getParameterObject().getParameterName();

	var active = SemwidgJS.SparqlConnection._isParameterObjectPartActive(widget, property, parameterObjectPart);

	if (active && part && (part.getType() == 'semwidgql' || part.getType() == 'sparql')
			&& part.getQueryString()) {

		var queries = part.getQueryString();
	
		var endpoint = SemwidgJS.SparqlConnection._findEndpoint(widget, property, part);

		var refreshSchedule = SemwidgJS.SparqlConnection._getRefreshInterval(widget, property, part);
		later.date.localTime();

		var skipLoading = false;
		if (part.isLoading == true) {
			if (SemwidgJS.SparqlConnection.loadingSince[part.id]) {
				var n = Date.now();
				var s = SemwidgJS.SparqlConnection.loadingSince[part.id];

				if (n - s < 500) {
					skipLoading = true;
				}
			}
		}

		var url;

		if (endpoint && !skipLoading) {
			var proxy = endpoint.getProxy();

			if (SemwidgJS.Util.String.isEmpty(proxy)) {
				proxy = 'none';
			}

			if (proxy.toLowerCase() != 'none') {

				if (proxy.toLowerCase() == 'default') {
					url = SemwidgJS.Util.getDefaultProxy() + '?url=' + endpoint.getUrl();
				}else {
					var proxyObj = SemwidgJS.getConfig().getProxy(proxy);
					if (!proxyObj) {
						console.error('Proxy "' + proxy + '" is missing!');
						return;
					}
					url = SemwidgJS.getConfig().getProxy(proxy).getUrl() + '?url=' + endpoint.getUrl();
				}
				
			}else {
				url = endpoint.getUrl();
			}
		}

		if (url && queries && !skipLoading) {
			
			var method = ('' + endpoint.getHttpMethod()).toUpperCase();

			var query;

			for (var q = 0; q < queries.length; q++) {
			
				query = queries[q];

				if (method == 'GET') {
					if (proxy.toLowerCase() == 'none') {
						url += '?';
					}
					url += '&format=XML';
					url += '&output=XML';
					if (!SemwidgJS.Util.String.isEmpty(endpoint.getDefaultGraphUri())){
						url += '&default-graph-uri=' + encodeURIComponent(endpoint.getDefaultGraphUri());
					}
					url += '&query=' + encodeURIComponent(query);
				}

				var cacheDuration = SemwidgJS.SparqlConnection._getCacheDuration(widget, property, part);

				// check SparqlCache
				var cacheResult = SemwidgJS.SparqlCache.getResult(
						query, 
						endpoint.getUrl(), 
						endpoint.getDefaultGraphUri(), 
						'XML'
					);

				var useCache = true;

				var now = Date.now();
				
				if (cacheResult && cacheResult.getStatus() > 0) {
					var lastUpdate = cacheResult.getLastUpate().getTime();

					var nextUpdate = lastUpdate + (cacheDuration * 1000);
					
					if (nextUpdate < now) {

						useCache = false;

					} else if (refreshSchedule) {
						var previousUpdate = later.schedule(refreshSchedule).prev(1);

						if (previousUpdate && lastUpdate < previousUpdate.getTime()) {
							useCache = false;
						}
					}
				}

				if (useCache && cacheResult && cacheResult.getStatus() > 0) {
					
					part.isLoading = false;

					var result = new SemwidgJS.SparqlResult();
					result.setXmlResult(cacheResult.getResult());
					part.setSparqlResult(result, q);

				// if cache is empty -> request
				}else {
					
					var doRequest = function(part, method, index) {
						part.isLoading = true;

						SemwidgJS.SparqlConnection.loadingSince[part.id] = now;

						var request;

						if (method == 'GET') {
							request = $.get(url);
						}else {
							request = $.post(
								url,
								{
									'format': 'XML',
									'output': 'XML',
									'default-graph-uri': endpoint.getDefaultGraphUri(),
									'query': query
								}
							);
						}
						
						request._swSparqlQuery = query;
						request._swEndpointUrl = endpoint.getUrl();
						request._swDefaultGraphUri = endpoint.getDefaultGraphUri();
						request._swFormat = 'XML';

						request.complete(
							function(data) {

								part.isLoading = false;

								if (data.status == 200) {
									SemwidgJS.SparqlCache.insertResult(
											data._swSparqlQuery,
											data._swEndpointUrl,
											data._swDefaultGraphUri,
											data._swFormat,
											SemwidgJS.SparqlCache.ResultObject.STATUS_FOUND,
											data.responseText);

									var result = new SemwidgJS.SparqlResult();
									result.setXmlResult(data.responseText);
									part.setSparqlResult(result, index);
								}
							}
						);
					}(part, method, q);
				}
			}
		}

		if (refreshSchedule) {

			var timeout = SemwidgJS.SparqlConnection.timeouts[parameterObjectPart.getId()];

			if (timeout) {
				clearTimeout(timeout);
			}

			var nextUpdates = later.schedule(refreshSchedule).next(3);
			var nextUpdate;
			now = Date.now();
			for (var i = 0 ; i < nextUpdates.length; i++) {
				if (now < nextUpdates[i].getTime()) {
					nextUpdate = nextUpdates[i].getTime();
					break;
				}
			}

			SemwidgJS.SparqlConnection.timeouts[parameterObjectPart.getId()] = setTimeout(function() {
				SemwidgJS.SparqlConnection.requestDataForParameterObjectPart(parameterObjectPart);
			}, nextUpdate - now);
		}
	}

};

SemwidgJS.SparqlConnection._isParameterObjectPartActive = function(widget, property, parameterObjectPart) {

	var parameterObjectParts = widget._getParameterObjects()[property].parameterObjectParts;

	for (var i = 0; i < parameterObjectParts.length; i++) {
		if (parameterObjectParts[i] == parameterObjectPart) {
			return true;
		}
	}

	return false;
};

SemwidgJS.SparqlConnection._findEndpoint = function(widget, property, parameterObjectPart) {

	var endpoint;
	var endpointObject;
	var endpointUrl;

	// get endpoint from named resource
	if (parameterObjectPart.getType() == 'semwidgql') {
		var str = parameterObjectPart.getOriginalStringWithMapping();
		var semwidgQuery = str.substring(1, str.length - 1);
		var namedResources = SemwidgQL.getNamedResources(semwidgQuery);

		if (namedResources.length > 0) {

			var resource = SemwidgJS.getConfig().getResource(namedResources[0]);
			if (resource) {
				endpoint = resource.getEndpoint();
			}
		}
	}

	if (endpoint) {
		endpointObject = SemwidgJS.getConfig().getEndpoint(endpoint);
		if (endpointObject) {
			return endpointObject;
		}
	}

	// get endpoint from property
	var endpointFunctionName = 'get' + SemwidgJS.Util.String.toUpperCamelCase(property) + 'Endpoint';
	endpoint = widget[endpointFunctionName].call(widget);

	if (endpoint) {
		endpointObject = SemwidgJS.getConfig().getEndpoint(endpoint);
		if (endpointObject) {
			return endpointObject;
		}

		// very simple check, if value could be a URL
		if (endpoint.indexOf(':') > 0) {
			endpointObject = new SemwidgJS.Endpoint();
			endpointObject.setUrl(endpoint);
			return endpointObject;
		}
	}


	// get default endpoint of widget
	endpoint = widget.getEndpoint();
	if (endpoint) {

		endpointObject = SemwidgJS.getConfig().getEndpoint(endpoint);
		if (endpointObject) {
			return endpointObject;
		}

		// very simple check, if value could be a URL
		if (endpoint.indexOf(':') > 0) {
			endpointObject = new SemwidgJS.Endpoint();
			endpointObject.setUrl(endpoint);
			return endpointObject;
		}
	}

	// get default endpoint
	var endpoints = SemwidgJS.getConfig().getEndpoints();
	for (var i = 0; i < endpoints.length; i++) {
		if (endpoints[i].getDefault()) {
			return endpoints[i];
		}
	}

	console.warn('No endpoint found for query ' 
			+ parameterObjectPart.getOriginalString() 
			+ ' in ' + widget.getId() );

	return;
};

SemwidgJS.SparqlConnection._getRefreshInterval = function(widget, property, part) {

	var interval;

	var functionName = 'get' + property + 'Refresh';

	if (typeof widget[functionName] === 'function') {
		interval = widget[functionName].call(widget);
	}

	if (SemwidgJS.Util.String.isEmpty(interval)) {
		interval = widget.getRefresh();
	}

	if (!SemwidgJS.Util.String.isEmpty(interval)) {

		var sched = later.parse.text(interval);

		if (sched.error >= 0) {
			sched = later.parse.cron(interval);
		}

		return sched;
	}

	return null;

};

SemwidgJS.SparqlConnection._getCacheDuration = function(widget, property, part) {

	var duration;

	var functionName = 'get' + property + 'Cache';

	if (typeof widget[functionName] === 'function') {
		duration = widget[functionName].call(widget);
	}

	if (SemwidgJS.Util.String.isEmpty(duration)) {
		duration = widget.getCache();
	}

	var seconds = 0;

	if (!SemwidgJS.Util.String.isEmpty(duration)) {

		var arr = duration.split(' ');
		
		var value;
		var unit;

		for (var i = 0; i < arr.length; i = i + 2) {
			
			value = parseFloat(arr[i]);
			if (isNaN(value)) {
				break;
			}

			// first char (d, h, m, s)
			unit = (i + 1 < arr.length) ? arr[i + 1][0] : 's';
			var mul = 0;

			switch(unit) {
			case 'd':
				mul = 86400;
				break;
			case 'h':
				mul = 3600;
				break;
			case 'm':
				mul = 60;
				break;
			case 's':
				mul = 1;
				break;
			}

			seconds += value * mul;
		}
	} else {
		seconds = 604800;
	}

	if (seconds < 10) {
		seconds = 10;
	}

	// 7 days
	if (seconds > 604800) {
		seconds = 604800;
	}

	return seconds;
};