SemwidgJS.SwOpenStreetMap = function() {
	this._addProperties([
		['lat', 'query', 'Latitude.'],
		['long', 'query', 'Longitude.'],
		['zoom', 'query', 'Zoom level.'],
		['area', 'query', 'Coordinates of an outline describing an area.']
	]);
};

SemwidgJS.SwOpenStreetMap.prototype = new SemwidgJS.SwWidget();

SemwidgJS.SwOpenStreetMap.L = null;

SemwidgJS.SwOpenStreetMap.libsAdded = false;

SemwidgJS.SwOpenStreetMap.buildQueue = [];

SemwidgJS.SwOpenStreetMap.addToBuildQueue = function(buildFunction) {
	SemwidgJS.SwOpenStreetMap.buildQueue.push(buildFunction);
};

SemwidgJS.SwOpenStreetMap.executeBuildQueue = function() {
	var buildFunction;

	while (SemwidgJS.SwOpenStreetMap.buildQueue.length > 0) {
		buildFunction = SemwidgJS.SwOpenStreetMap.buildQueue.pop();
		buildFunction();
	}
};

SemwidgJS.SwOpenStreetMap.prototype._leafletmap;

SemwidgJS.SwOpenStreetMap.prototype._destroy = function() {
	if (this._leafletmap) {
		this._leafletmap.remove();
		this._leafletmap = null;
		$('#' + this.__getUID() + '_leafletmap').remove();
	}
};

SemwidgJS.SwOpenStreetMap.prototype._buildHtml = function() {

	var leafletJsUrl = 'https://cdnjs.cloudflare.com/ajax/libs/leaflet/0.7.7/leaflet.js';
	var leafletCssUrl = 'https://cdnjs.cloudflare.com/ajax/libs/leaflet/0.7.7/leaflet.css';

	var self = this;

	self._destroy();

	var buildHtml = function() {

		var html = '';

		html += '<div id="' + self.__getUID() + '_leafletmap' + '" ';
		html += self._getDefaultHtmlAttributes();
		html += '>';
		html += '</div>';

		self._setHtml(html);

		var htmlElement = $('#' + self.__getUID() + '_leafletmap');

		// default height
		if (htmlElement.height() == 0) {
			htmlElement.height(300);
		}

		var lat = parseFloat(self.getLatDefaultValue());
		var lon = parseFloat(self.getLongDefaultValue());
		var zoom = parseInt(self.getZoomDefaultValue());

		if (isNaN(zoom)) {
			zoom = 2;
		}

		var area;
		var areaJv = self.getAreaJoinedValues();

		// prepare area
		if (areaJv) {
			area = [];
			for (var i = 0; i < areaJv.length; i++) {
				
				var areaRs = areaJv[i][0].getResults();
				
				for (var j = 0; j < areaRs.length; j++) {
					
					var l = areaRs[j].length - 1;
					var outline = areaRs[j][l].getValue();

					try {
						var parsed = JSON.parse('[' + outline  + ']');
						area.push(parsed);
					}catch (e) {
						console.error(e);
					}
				}
			}
		}


		if (!isNaN(lat) && !isNaN(lon) || area) {

			try {

				if (!SemwidgJS.SwOpenStreetMap.L) {
					SemwidgJS.SwOpenStreetMap.L = L;
				}
				
				self._leafletmap = SemwidgJS.SwOpenStreetMap.L.map(self.__getUID() + '_leafletmap').setView(
					[lat, lon], zoom);

				// add an OpenStreetMap tile layer
				SemwidgJS.SwOpenStreetMap.L.tileLayer('https://{s}.tile.openstreetmap.org/{z}/{x}/{y}.png', {
				    attribution: '&copy; <a href="https://openstreetmap.org/copyright">OpenStreetMap</a> contributors'
				}).addTo(self._leafletmap);

				// add a marker at the given location
				SemwidgJS.SwOpenStreetMap.L.marker([lat, lon]).addTo(self._leafletmap);

				// add a area if polygons is defined
				if (area && area.length > 0) {
					SemwidgJS.SwOpenStreetMap.L.multiPolygon(area).addTo(self._leafletmap);
				}


			}catch (e) {
				console.error(e);
			}
		}
	};

	var jsAddedHandler = function() {
		SemwidgJS.SwOpenStreetMap.executeBuildQueue();
		buildHtml();
	};

	var cssAddedHandler = function() {
		SemwidgJS.Util.addJS(leafletJsUrl, '_leafletJS', jsAddedHandler);
	};

	var addLeaflet = function() {
		SemwidgJS.SwOpenStreetMap.libsAdded = true;
		SemwidgJS.Util.addCSS(leafletCssUrl, '_leafletCSS', cssAddedHandler);
	};

	if (SemwidgJS.SwOpenStreetMap.L) {
		SemwidgJS.SwOpenStreetMap.libsAdded = true;
	}


	// TODO: Not working, when Leaflet.js is already loading
	if (!SemwidgJS.SwOpenStreetMap.L && !SemwidgJS.SwOpenStreetMap.libsAdded) {
		addLeaflet();
	}else if (!SemwidgJS.SwOpenStreetMap.L && SemwidgJS.SwOpenStreetMap.libsAdded) {
		SemwidgJS.SwOpenStreetMap.addToBuildQueue(buildHtml);
	}else {
		buildHtml();
	}

};

SemwidgJS.WidgetProvider.registerWidgetType('SemwidgJS.SwOpenStreetMap');
