SemwidgJS.SwTable = function() {

	this._addProperties([
		['header-index', 'static', 'default = last'],
		['data-indices', 'static', 'Comma-separated list of indices, default = last - 1'],
		['data-labels', 'query', 'Comma-separated list of labels'],
		['pivot', 'static', 'default = false']
	]);
};

SemwidgJS.SwTable.prototype = new SemwidgJS.SwWidget();

SemwidgJS.SwTable.prototype._buildHtml = function() {

	var html = '';
	
	if (!this._isLoading()) {

		var joinedValues = this.getValueJoinedValues();

		var mathParser = new SemwidgJS.Util.MathParser();

		var pivot = false;

		var i, j, k;

		if (!SemwidgJS.Util.String.isEmpty(this.getPivot())) {
			var sl = this.getPivot().trim().toLowerCase();
			if (sl == 'true' || sl == 'yes' || sl == '1') {
				pivot = true;
			}else {
				pivot = false;
			}
		}

		var headerIndex = this.getHeaderindex();
		if (SemwidgJS.Util.String.isEmpty(headerIndex)) {
			headerIndex = 'last';
		}

		var dataIndices = [];
		if (!SemwidgJS.Util.String.isEmpty(this.getDataindices())) {
			var di = this.getDataindices();
			dataIndices = di.split(',');
			for (i = 0; i < dataIndices.length; i++) {
				dataIndices[i] = '' + dataIndices[i].trim().toLowerCase();
			}
		}else {
			dataIndices.push('last - 1');
		}
		
		var headers = [];
		var dataRows = [];

		for (k = 0; k < dataIndices.length; k++) {
			dataRows.push([]);
		}

		var labels = this.getDatalabelsDefaultValue();
		if (!SemwidgJS.Util.String.isEmpty(labels)) {
			labels = labels.split(',');
		}

		for (i = 0; i < joinedValues.length; i++) {

			var jv = joinedValues[i][0];

			var max = jv.getVariableNames().length - 1;

			if (headerIndex.toLowerCase() != 'none') {
				headerIndex = mathParser.parse(('' + headerIndex).toLowerCase().replace('first', 0).replace('last', max));

				if (headerIndex > max) {
					console.warn('Header-Index out of bounds. Max value is ' + max + '.');
					headerIndex = max;
				}

				if (headerIndex < 0) {
					headerIndex = 0;
				}
			}

			for (k = 0; k < dataIndices.length; k++) {
				dataIndices[k] = mathParser.parse(('' + dataIndices[k]).toLowerCase().replace('first', 0).replace('last', max));
				if (dataIndices[k] > max) {
					console.warn('Data-Index out of bounds. Max value is ' + max + '.');
					dataIndices[k] = max;
				}

				if (dataIndices[k] < 0) {
					dataIndices[k] = 0;
				}
			}

			var result;
			for (j = 0; j < jv.getResults().length; j++) {
				result = jv.getResults()[j];

				if (('' + headerIndex).toLowerCase() != 'none') {
					headers.push(result[headerIndex].getValue());
				}

				for (k = 0; k < dataIndices.length; k++) {
					dataRows[k].push(result[dataIndices[k]].getValue());
				}
			}

		}

		html += '<table ';
		html += this._getDefaultHtmlAttributes();
		html += ' border="1" cellpadding="4">';

		if (!pivot) {

			if (headers.length > 0) {
				html += '<tr>';

				if(labels) {
					html += '<th></th>';
				}

				for (i = 0; i < headers.length; i++) {
					html += '<th>' + headers[i] + '</th>';
				}

				html += '</tr>';
			}
				
			var dataRow;
			var align;
			for (i = 0; i < dataRows.length; i++) {
				dataRow = dataRows[i];

				html += '<tr>';

				if(labels) {
					if (labels.length > i) {
						html += '<th>' + labels[i] + '</th>';
					}else {
						html += '<th></th>';
					}
					
				}

				for (j = 0; j < dataRow.length; j++) {
					
					if (isNaN(dataRow[j])) {
						align = 'left';
					}else {
						align = 'right';
					}

					html += '<td style="text-align: ' + align + '">' + dataRow[j] + '</td>';
				}
				html += '</tr>';
			}

		}else {

			if (labels) {
				html += '<tr>';
				if (headers.length > 0) {
					html += '<th></th>';
				}

				for (i = 0; i < labels.length; i++) {
					html += '<th>' + labels[i] + '</th>';
				}

				html += '</tr>';
			}

			if (dataRows.length > 0) {
				for (i = 0; i < dataRows[0].length; i++) {

					html += '<tr>';

					if (headers.length > 0) {
						if (headers.length > i) {
							html += '<th>' + headers[i] + '</th>';
						}else {
							html += '<th></th>';
						}
					}

					for (j = 0; j < dataRows.length; j++) {
						dataRow = dataRows[j];

						if (isNaN(dataRow[i])) {
							align = 'left';
						}else {
							align = 'right';
						}

						html += '<td style="text-align: ' + align + '">' + dataRow[i] + '</td>';
					}

					html += '</tr>';

				}
			}

		}

		html += '</table>';

	}

	this._setHtml(html);
};

SemwidgJS.WidgetProvider.registerWidgetType('SemwidgJS.SwTable');
