/**
 * Semwidg SwWidget
 *
 * Base Widget 'Class'. All other widgets inherit from SwWidget.
 */

SemwidgJS.SwWidget = function() {

//TODO: add enabled property

	this._addProperties([
		['id', 'fixed', 'Unique identifier in SemwidgJS and HTML.'],
		['endpoint', 'static', 'Endpoint to use, if neither query nor value property define an endpoint.'],
		['value', 'query', 'Value to display in HTML.'],
		['styleclass', 'query', 'CSS class name(s) applied to HTML.'],
		['style', 'query', 'CSS styles applied to HTML.'],
		['title', 'query', 'Title attribute applied to HTML, typically mouse over text.'],
		['other', 'query', 'All other possible HTML attributes. Value of "other" will be inserted into HTML as it is entered.'],
		['refresh', 'static', 'Refresh Interval for widgets.'],
		['limit', 'static', 'Number of maximum results for a query.'],
		['cache', 'static', 'The duration query results will be cached. Minimum: 10 seconds, Maximum: 7 days.']
	]);

};

SemwidgJS.SwWidget.prototype = new SemwidgJS.HtmlElement();

SemwidgJS.SwWidget.prototype._addProperties = function(properties) {
	if (!this.properties) {
		this.properties = {};
	}

	var addedProperties = [];

	for (var i = 0; i < properties.length; i++) {

		if (properties[i] instanceof Array && properties[i].length == 3
				&& !SemwidgJS.Util.String.isEmpty(properties[i][0])) {

			var propertyType = properties[i][1].toLowerCase();

			properties[i][0] = SemwidgJS.Util.String.toUpperCamelCase(
					SemwidgJS.Util.String.toUpperCamelCase(properties[i][0]));
			if (propertyType !== 'fixed' && propertyType !== 'static' && propertyType !== 'selector') {
				propertyType = 'query';
			}
			
			properties[i][1] = propertyType;
			this.properties[properties[i][0]] = properties[i];

			addedProperties.push(properties[i])
		}
	}

	this._dispatchEvent(new SemwidgJS.Event('propertiesAdded', addedProperties));
};

SemwidgJS.SwWidget.prototype._getProperties = function() {
	return this.properties;
};

SemwidgJS.SwWidget.prototype._getQueryProperties = function() {

	var queryProperties = [
		['Endpoint', 'static', 'Endpoint to use, if query does not define an endpoint.'],
		['Order', 'static', 'Comma-separated list of indices and optional order direction. Example: last - 1 asc, last desc'],
		['Selector', 'static', 'A selector function.'],
		['Transform', 'static', 'A transform function.'],
		['Refresh', 'static', 'Refresh Interval for widget property.'],
		['Limit', 'static', 'Number of maximum results for a query.'],
		['Cache', 'static', 'The duration query results will be cached. Minimum: 1 minute, Maximum: 7 days.']
	];

	return queryProperties;

};

SemwidgJS.SwWidget.prototype._getParameterObjects = function() {
	if (!this.parameterObjects) {
		this.parameterObjects = {};
	}

	return this.parameterObjects;
};

SemwidgJS.SwWidget.prototype._addParameterObject = function(name, value) {

//TODO: remove eventlistener from old parameter object
	var parameterObject = new SemwidgJS.ParameterObject();

	this._getParameterObjects()[name] = parameterObject;

	parameterObject.setWidget(this);
	parameterObject.setParameterName(name);
	parameterObject.setOriginalString(value);

	// add event listener
	var owner = this;
	var handleParameterObjectPartsChanged = function(event) {
		this.owner = owner;
		this.owner._build();
	};
	parameterObject._addEventListener('resultsChanged', handleParameterObjectPartsChanged, this);

	parameterObject.process();

};

SemwidgJS.SwWidget.prototype._isLoading = function() {

	var pos = this._getParameterObjects();
	
	if (!pos) {
		return false;
	}

	for (var key in pos) {
		if (pos[key] && pos[key].isLoading()) {
			return true;
		}
	}

	return false;

};

SemwidgJS.SwWidget.prototype._build = function() {
	
	try {

		if (this._isLoading()){
			this._displayLoadingAnimation();
		}else {
			this._orderAll();
			this._buildHtml();
		}

	}catch (e) {
		console.error(e);
	}
	
};

SemwidgJS.SwWidget.prototype._buildHtml = function() {
	// overide with specific code for widget
};

SemwidgJS.SwWidget.prototype._destroy = function() {
	// override with specific code for widget
};

SemwidgJS.SwWidget.prototype._getDefaultHtmlAttributes = function() {

	var attributes = ' ';

	if (!SemwidgJS.Util.String.isEmpty(this.getStyleclass())) {
		attributes += 'class="' + SemwidgJS.Util.String.htmlEscape(this.getStyleclassDefaultValue()) + '" ';
	}

	if (!SemwidgJS.Util.String.isEmpty(this.getStyle())) {
		attributes += 'style="' + SemwidgJS.Util.String.htmlEscape(this.getStyleDefaultValue()) + '" ';
	}

	if (!SemwidgJS.Util.String.isEmpty(this.getTitle())) {
		attributes += 'title="' + SemwidgJS.Util.String.htmlEscape(this.getTitleDefaultValue()) + '" ';
	}

	if (!SemwidgJS.Util.String.isEmpty(this.getOtherDefaultValue())) {
		attributes += this.getOther() + ' ';
	}

	return attributes;
}

SemwidgJS.SwWidget.prototype._setHtml = function(html) {
	this._html = html;

	var node = $(this._getNode());

	node.empty();
	node.append(this._html);

	this._dispatchEvent(new SemwidgJS.Event('htmlChanged', this._html));
	this._dispatchEvent(new SemwidgJS.Event('widgetChanged', 'html'));
};

SemwidgJS.SwWidget.prototype._updateHtml = function(attribute, value){

	var node = $(this._getNode()).children().first();

	node.attr(attribute, value);
};

SemwidgJS.SwWidget.prototype._orderAll = function() {

	for (var property in this._getProperties()) {
		if(('' + this._getProperties()[property][1]).toLowerCase() == 'query') {
			this._order(property);
		}
	}

};

SemwidgJS.SwWidget.prototype._order = function(propertyName) {

	var propertyNameCapital = SemwidgJS.Util.String.toUpperCamelCase(propertyName);

	var getOrder = 'get' + propertyNameCapital + 'Order';
	var order = this[getOrder]();
	var orderObjects = [];

	if(SemwidgJS.Util.String.isEmpty(order)) {
		order = 'last';
	}else if (order.trim().toLowerCase() == 'none') {
		return;
	}

	if (!SemwidgJS.Util.String.isEmpty(order)) {
		var orderArray = ('' + order).split(',');

		// parse order attributes
		for (var i = 0; i < orderArray.length; i++) {
			
			var o = orderArray[i].trim().split(/\s+/);
			var orderObject = {index: 0, direction: 0};
			
			if (o.length > 1) {
				var direction = (o[o.length - 1]).toLowerCase();
				if (direction == 'asc') {
					orderObject.direction = 0;
					o.pop();
				}else if (direction == 'desc') {
					orderObject.direction = 1;
					o.pop();
				}else {
					orderObject.direction = 0;
				}
			}

			var getAllValues = 'get' + propertyNameCapital + 'AllValues';
			var allValues = this[getAllValues]();
			var mathParser = new SemwidgJS.Util.MathParser();

			for (var j = 0; j < allValues.length; j++) {

				var av = allValues[j];
				var max = av.getVariableNames().length - 1;

				var orderIndex = o.join(' ');
				
				orderObject.index = mathParser.parse(('' + orderIndex).toLowerCase().replace('first', 0).replace('last', max));
				
				if (isNaN(orderObject.index)) {
					console.warn('Order index "' + orderIndex + '" is invalid.');
					return;
				}
			}

			orderObjects.push(orderObject);
		}

		var parameterObject = this._getParameterObjects()[propertyNameCapital];

		if (!parameterObject) {
			return;
		}

		var parts = parameterObject.getParameterObjectParts();

		// sort
		for (var i = 0; i < parts.length; i++) {
			if (parts[i] && parts[i].results && parts[i].results[0] && parts[i].results[0].resultObject) {
				var toSort = parts[i].results[0].resultObject.results;

				toSort.sort(function(a, b) {

					for (var j = 0; j < orderObjects.length; j++) {
						var orderObject = orderObjects[j];
						
						var aVal = (a[orderObject.index]) ? a[orderObject.index].value : '';
						
						if (!isNaN(parseFloat(aVal)) && parseFloat(aVal).toString() == aVal) {
							aVal = aVal * 1;
						}else if (moment(aVal, moment.ISO_8601).isValid()) {
							aVal = moment(aVal, moment.ISO_8601);
						}else {
							aVal = aVal.toLowerCase();
						}

						var bVal = (b[orderObject.index]) ? b[orderObject.index].value : '';
						
						if (!isNaN(parseFloat(bVal)) && parseFloat(bVal).toString() == bVal) {
							bVal = bVal * 1;
						}else if (moment(bVal, moment.ISO_8601).isValid()) {
							bVal = moment(bVal, moment.ISO_8601);
						}else {
							bVal = bVal.toLowerCase();
						}

						if (aVal < bVal) {
							if (orderObject.direction == 1) {
								return 1;
							}else {
								return -1;
							}
						}
						if (aVal > bVal) {
							if (orderObject.direction == 1) {
								return -1;
							}else {
								return 1;
							}
						}
					}
					
					return 0;
				});
			}
		}
	}
};

SemwidgJS.SwWidget.prototype._displayLoadingAnimation = function() {

	var text = '<span class="semwidgjs-loading-animation" title="Loading...">';
	text += '<span style="color: #ff7200;">■</span>';
	text += '<span style="color: #d5007c;">■</span>';
	text += '<span style="color: #86ba0c;">■</span>';
	text += '</span>';
	this._setHtml(text);

	var x = 0;
	var node = this._getNode().firstChild;
	var self = this;
	
	var animate = function() {

		if (!self._isLoading()){
			clearInterval(animationId) ;
		}

		node.insertBefore(node.childNodes[2], node.childNodes[0]);
	};

	var animationId = setInterval(animate, 250);
};

SemwidgJS.WidgetProvider.registerWidgetType('SemwidgJS.SwWidget');
