SemwidgJS.Template = function() {

};

SemwidgJS.Template.prototype = new SemwidgJS.HtmlElement();

SemwidgJS.Template.prototype._getType = function() {
	if (this._type) {
		return this._type;
	}

	return 'SwTemplate';
};

SemwidgJS.Template.prototype._setType = function(type) {
	this._type = type;

	// set Template Definition
	var tp = SemwidgJS.getTemplateProvider();
	var definition = tp.getTemplateDefinition(this._getType());
	this._setTemplateDefinition(definition);
};

SemwidgJS.Template.prototype._getTemplateDefinition = function() {
	return this._templateDefinition;
};

SemwidgJS.Template.prototype._setTemplateDefinition = function(templateDefinition) {
	this._templateDefinition = templateDefinition;

	// handle TemplateChanged events
	if (this._templateDefinition) {

		var owner = this;
		
		var handleTemplateChangedEvent = function(event) {

			// owner.clearOldTemplateDefinition(); TODO: implement

			owner._build();
			SemwidgJS.getWidgetProvider().readWidgets(owner._getNode());

		};

		this._templateDefinition._addEventListener('templateDefinitionChanged', handleTemplateChangedEvent, this);

	}
};

SemwidgJS.Template.prototype.removeWidgets = function() {
	// delete old widgets
	var allWidgets = SemwidgJS.getWidgetProvider().getWidgets();
	var templateNode = $(this._getNode());
	for (var widgetId in allWidgets) {
		var widget = allWidgets[widgetId];
		var widgetNode = $(widget._getNode());

		// is templateNode ancestor of widgetNode?
		if (widgetNode.closest(templateNode).length > 0) {
			SemwidgJS.getWidgetProvider().removeWidget(widget);
		}
	}
};

SemwidgJS.Template.prototype.getParameters = function() {
	if (!this.parameters) {
		this.parameters = {};
	}

	return this.parameters;
};

SemwidgJS.Template.prototype.get = function(parameter) {

	return this.getParameters()[parameter];
};

SemwidgJS.Template.prototype.set = function(parameter, value) {

	this.getParameters()[parameter] = value;

};

SemwidgJS.Template.prototype._build = function() {

	this._buildHtml();

};

SemwidgJS.Template.prototype._buildHtml = function() {

	var definition = this._getTemplateDefinition();
	
	if (!definition) {
		console.error('TemplateDefinition "' + this._getType() + '" for "' + this.getId() + '" is unkown.')
		return;
	}

	var html = definition.getHtmlContent();

	for (var i = 0; i < definition.getParameters().length; i++) {
		var parameter = definition.getParameters()[i];

		if (SemwidgJS.Util.String.isEmpty(parameter)) {
			continue;
		}

		var value = this.get(parameter);

		if (SemwidgJS.Util.String.isEmpty(value)) {
			continue;
		}

		html = SemwidgJS.Util.String.replaceAll(html, '%%' + parameter + '%%', value);
	}

	this._setHtml(html);

};

SemwidgJS.Template.prototype._setHtml = function(html) {
	this._html = html;

	var node = $(this._getNode());

	node.empty();
	node.append(this._html);

	this._dispatchEvent(new SemwidgJS.Event('htmlChanged', this._html));
	this._dispatchEvent(new SemwidgJS.Event('templateChanged', 'html'));
};

SemwidgJS.TemplateDefinition = function() {

};

SemwidgJS.TemplateDefinition.prototype = new SemwidgJS.HtmlElement();

SemwidgJS.TemplateDefinition.prototype._getType = function() {
	return 'SwTemplateDefinition';
};

// this.parameter holds the whole Sting that is used inside the html defintion
SemwidgJS.TemplateDefinition.prototype.getParameter = function() {
	return this.parameter;
};

SemwidgJS.TemplateDefinition.prototype.setParameter = function(parameter) {

	if (SemwidgJS.Util.String.isEmpty(parameter)) {
		console.warn('TemplateDefinition "' + this.getId() + '": parameter is empty');
	}

	this.parameter = parameter;

	if (this.parameter !== parameter) {
		this.parameter = parameter;

		this._writeToDom('parameter');

		this._dispatchEvent(new SemwidgJS.Event('parameterChanged', this.parameter));
		// this._dispatchEvent(new SemwidgJS.Event('templateDefinitionChanged', 'parameter'));
	}

	var parameters = [];

	if (!SemwidgJS.Util.String.isEmpty(parameter)) {
		parameters = parameter.split(',');
		for (var i = 0; i < parameters.length; i++) {
			parameters[i] = parameters[i].trim().replace(/[\%]/g, '');
		}
	}

	// parameter 'template' is always available and returns the id of the template
	parameters.push('template');

	this.setParameters(parameters);
};

// this.parameters holds an array of single parameters defined by this.parameter
SemwidgJS.TemplateDefinition.prototype.getParameters = function() {
	return this.parameters;
};

SemwidgJS.TemplateDefinition.prototype.setParameters = function(parameters) {

	if (this.parameters !== parameters) {
		this.parameters = parameters;

		this._dispatchEvent(new SemwidgJS.Event('parametersChanged', this.parameters));
		this._dispatchEvent(new SemwidgJS.Event('templateDefinitionChanged', 'parameters'));
	}

};

SemwidgJS.TemplateDefinition.prototype.getHtmlContent = function() {
	return this.htmlContent;
};

SemwidgJS.TemplateDefinition.prototype.setHtmlContent = function(htmlContent) {
	
	if (SemwidgJS.Util.String.isEmpty(htmlContent)) {
		console.warn('TemplateDefinition "' + this.getId() + '": htmlContent is empty');
	}

	if (this.htmlContent !== htmlContent) {
		this.htmlContent = htmlContent;
		
		this._dispatchEvent(new SemwidgJS.Event('htmlContentChanged', this.htmlContent));
		this._dispatchEvent(new SemwidgJS.Event('templateDefinitionChanged', 'htmlContent'));
	}

};

SemwidgJS.TemplateDefinition.prototype._setNode = function(node) {
	
	// disconnect Mutation Observer form previous node
	if (this._node) {
		this._getMutationObserver().disconnect();
	}

	this._node = node;

	// observe attribute changes
	if (this._node) {
		var options = {attributes: true, subtree: true, childList: true, characterData: true};
		this._getMutationObserver().observe(this._node, options);
	}
};

// Overriding Mutation Observer for additional observation of Html content
SemwidgJS.TemplateDefinition.prototype._getMutationObserver = function() {
	
	if (!this._observer) {

		var owner = this;

		this._observer = new MutationObserver(function (mutations) {
			mutations.forEach(function (mutation) {

				this.owner = owner;

				if (mutation.target == this.owner._getNode() && mutation.type == 'attributes') {
					if (mutation.attributeName && SemwidgJS.Util.String.startsWith(
							mutation.attributeName, 'data-swp-')) {

						var attribute = mutation.attributeName.slice(5);

						//clear jQuery's data cache for this attribute
						$(mutation.target).removeData(attribute);
						
						var value = SemwidgJS.Util.String.htmlUnescape($(mutation.target).data(attribute));
						var propertyName = SemwidgJS.Util.String.toUpperCamelCase(mutation.attributeName.slice(9));
						var functionName = 'set' + propertyName;

						if (typeof this.owner[functionName] === 'function') {
							this.owner[functionName].call(this.owner, value);
						}else {
							console.warn('Property "' + propertyName + '" is not available for ' + this.owner.getId());
						}
					}
				}else {

					// if (mutation.type == 'characterData') {
					// 	var node = this.owner._getNode();
					// 	this.owner.setHtmlContent($(node).html());
					// }
					
				}
			})
		});
	}

	return this._observer;
};