/**
 * Semwidg Template Provider
 * 
 * stores all templates and it provides template related functions
 */

SemwidgJS.TemplateProvider = function() {

};

SemwidgJS.TemplateProvider.prototype = new SemwidgJS.EventDispatcher();

SemwidgJS.TemplateProvider.prototype.addTemplate = function(template) {
	
	if (template) {
		var t = this.getTemplate(template.getId());
		if (t) {
			this.removeTemplate(t);
		}

		this.getTemplates().push(template);
	}
};

SemwidgJS.TemplateProvider.prototype.getTemplate = function(id) {
	for (var i = 0; i < this.getTemplates().length; i++) {
		if (this.getTemplates()[i].getId() == id) {
			return this.getTemplates()[i];
		}
	}
	return null;
};

SemwidgJS.TemplateProvider.prototype.getTemplates = function() {
	if (!this.templates) {
		this.templates = [];
	}
	return this.templates;
};

SemwidgJS.TemplateProvider.prototype.removeTemplate = function(template) {
	var templates = this.getTemplates();

	SemwidgJS.Util.destroy(template);

	var index = -1;

	for (var i = 0; i < templates.length; i++) {
		if (templates[i] == template) {
			index = i;
			break;
		}
	}

	templates[i] = null;

	if (index > -1) {
		templates.splice(index, 1);
	}
};

SemwidgJS.TemplateProvider.prototype.getTemplateDefinition = function(id) {
	for (var i = 0; i < this.getTemplateDefinitions().length; i++) {
		if (this.getTemplateDefinitions()[i].getId() == id) {
			return this.getTemplateDefinitions()[i];
		}
	}
	return null;
};

SemwidgJS.TemplateProvider.prototype.getTemplateDefinitions = function() {
	if (!this.templateDefinitions) {
		this.templateDefinitions = [];
	}
	return this.templateDefinitions;
};

SemwidgJS.TemplateProvider.prototype.removeTemplateDefinition = function(templateDefinition) {
	var templateDefinitions = this.getTemplateDefinitions();

	SemwidgJS.Util.destroy(templateDefinition);

	var index = -1;

	for (var i = 0; i < templateDefinitions.length; i++) {
		if (templateDefinitions[i] == templateDefinition) {
			index = i;
			break;
		}
	}

	templateDefinitions[i] = null;

	if (index > -1) {
		templateDefinitions.splice(index, 1);
	}
};

SemwidgJS.TemplateProvider.prototype.registerTemplateDefinition = function(node) {

	var nodeObj = $(node);

	if (nodeObj.length == 0) {
		return;
	}

	// hide Template Defintions
	nodeObj.hide();

	var templateDefinition = new SemwidgJS.TemplateDefinition();

	var id = nodeObj.data('swp-id');

	var td = this.getTemplate(id);
	if (td) {
		this.removeTemplateDefinition(td);
	}

	if (SemwidgJS.Util.String.isEmpty(id)) {
		templateDefinition._generateId();
	}else {
		templateDefinition.setId(id);
	}

	var parameterString = nodeObj.data('swp-parameter');
	templateDefinition.setParameter(parameterString);
	
	var htmlContent = nodeObj.html();
	templateDefinition.setHtmlContent(htmlContent);

	templateDefinition._setNode(node);

	this.getTemplateDefinitions().push(templateDefinition);

};

SemwidgJS.TemplateProvider.prototype.readTemplates = function() {

	var templateNodeList = $.find('[data-sw-template]');

	for (var i = 0; i < templateNodeList.length; i++) {

		var templateNode = $(templateNodeList[i]);

		var template = this.buildTemplateFromNode(templateNode);

		this.addTemplate(template);

		template._build();
	}

};

SemwidgJS.TemplateProvider.prototype.buildTemplateFromNode = function(node) {

	var template = new SemwidgJS.Template();
	
	var id = node.data('swp-id');

	if (SemwidgJS.Util.String.isEmpty(id)) {
		template._generateId();
	}else {
		template.setId(id);
	}

	var type = node.data('sw-template');

	if (!SemwidgJS.Util.String.isEmpty(type)) {
		template._setType(type);
	}

	template._setNode(node[0]);

	// all other parameters
	var attributes = node[0].attributes;

	for (var i = 0; i < attributes.length; i++) {

		var attribute = attributes[i];

		if (SemwidgJS.Util.String.startsWith(attribute.name, 'data-swp')) {
			var parameterName = attribute.name.slice(9);
			var value = SemwidgJS.Util.String.htmlUnescape(attribute.value);
			
			template.set(parameterName, value);
		}
	}

	// template 
	template.set('template', template.getId());

	return template;

};
