/**
 * Semwidg Utils
 *
 * Util functions
 */

SemwidgJS.Util = function() {

};

SemwidgJS.Util.getIdCounter = function() {
	if (this.idCounter === undefined) {
		this.idCounter = 1;
	}
	return this.idCounter++;
};

SemwidgJS.Util.getUniqueId = function() {
	return SemwidgJS.Util.getIdCounter() + '-' + SemwidgJS.getInstanceId();
};

SemwidgJS.Util.checkId = function(id) {

	var valid = true;

	if (id.indexOf('.') != -1) {
		valid = false;
	}

	if (valid) {
		valid = SemwidgJS.Util._checkIdProxy(id);
	}

	if (valid) {
		valid = SemwidgJS.Util._checkIdEndpoint(id);
	}

	if (valid) {
		valid = SemwidgJS.Util._checkIdResource(id);
	}

	if (valid) {
		valid = SemwidgJS.Util._checkIdNamespace(id);
	}

	if (valid) {
		valid = SemwidgJS.Util._checkIdMapping(id);
	}

	if (valid) {
		valid = SemwidgJS.Util._checkIdWidget(id);
	}

	return valid;

};

SemwidgJS.Util._checkIdProxy = function(id) {

	if (SemwidgJS.Util.String.isEmpty(id)) {
		return false;
	}

	for (var i = 0; i < SemwidgJS.getConfig().getProxies().length; i++) {
		if (SemwidgJS.getConfig().getProxies()[i].getId() == id) {
			return false;
		}
	}

	return true;
};

SemwidgJS.Util._checkIdEndpoint = function(id) {

	if (SemwidgJS.Util.String.isEmpty(id)) {
		return false;
	}

	for (var i = 0; i < SemwidgJS.getConfig().getEndpoints().length; i++) {
		if (SemwidgJS.getConfig().getEndpoints()[i].getId() == id) {
			return false;
		}
	}

	return true;
};

SemwidgJS.Util._checkIdResource = function(id) {

	if (SemwidgJS.Util.String.isEmpty(id)) {
		return false;
	}

	for (var i = 0; i < SemwidgJS.getConfig().getResources().length; i++) {
		if (SemwidgJS.getConfig().getResources()[i].getId() == id) {
			return false;
		}
	}

	return true;
};

SemwidgJS.Util._checkIdNamespace = function(id) {

	if (SemwidgJS.Util.String.isEmpty(id)) {
		return false;
	}

	for (var i = 0; i < SemwidgJS.getConfig().getNamespaces().length; i++) {
		if (SemwidgJS.getConfig().getNamespaces()[i].getId() == id) {
			return false;
		}
	}

	return true;
};


SemwidgJS.Util._checkIdMapping = function(id) {

	if (SemwidgJS.Util.String.isEmpty(id)) {
		return false;
	}

	for (var i = 0; i < SemwidgJS.getConfig().getMappings().length; i++) {
		if (SemwidgJS.getConfig().getMappings()[i].getId() == id) {
			return false;
		}
	}

	return true;
};

SemwidgJS.Util._checkIdWidget = function(id) {

	if (SemwidgJS.Util.String.isEmpty(id)) {
		return false;
	}

	var widgets = SemwidgJS.getWidgetProvider().getWidgets();
	var widgetId;
	var widget;

	for (widgetId in widgets) {
		widget = widgets[widgetId];
		if (widget.getId() == id) {
			return false;
		}
	}

	return true;
};

SemwidgJS.Util.getAllSemwidgElements = function() {
	var elems = [];

	var config = SemwidgJS.getConfig();

	elems = elems.concat(config.getProxies()).concat(config.getEndpoints()).
			concat(config.getResources()).concat(config.getNamespaces()).
			concat(config.getMappings()).concat(SemwidgJS.getWidgetProvider().getWidgets()).
			concat(SemwidgJS.getTemplateProvider().getTemplateDefinitions()).
			concat(SemwidgJS.getTemplateProvider().getTemplates());

	return elems;
};


SemwidgJS.Util.getSemwidgElementById = function(id) {

	var elem;

	var config = SemwidgJS.getConfig();
	elem = config.getProxy(id);

	if (!elem) {
		elem = config.getEndpoint(id);
	}
	if (!elem) {
		elem = config.getResource(id);
	}
	if (!elem) {
		elem = config.getNamespace(id);
	}
	if (!elem) {
		elem = config.getMapping(id);
	}

	if (!elem) {
		elem = SemwidgJS.getWidgetProvider().getWidget(id);
	}

	if (!elem) {
		elem = SemwidgJS.getTemplateProvider().getTemplateDefinition(id);
	}

	if (!elem) {
		elem = SemwidgJS.getTemplateProvider().getTemplate(id);
	}

	return elem;
};

SemwidgJS.Util.getValueBySelector = function(selector) {

	var parts = selector.split('.');
	var result;

	if (parts.length == 1) {
		result = SemwidgJS.Util.getSemwidgElementById(id);
	}else if (parts.length == 2) {
		result = SemwidgJS.Util.getValue(parts[0], parts[1]);
	}

	if (result) {
		return result;
	}else {
		console.warn('Selector "' + selector + '" is invalid.');
	}

};

SemwidgJS.Util.setValueBySelector = function(selector, value) {

	var parts = selector.split('.');
	var result;

	if (parts.length == 2) {
		result = SemwidgJS.Util.setValue(parts[0], parts[1], value);
	}

	if (result) {
		return result;
	}else {
		console.warn('Selector "' + selector + '" is invalid.');
	}

};

SemwidgJS.Util.getValue = function(id, propertyName) {

	var elem = SemwidgJS.Util.getSemwidgElementById(id);
	var functionName = 'get' + SemwidgJS.Util.String.toUpperCamelCase(propertyName);

	if (elem && typeof elem[functionName] === 'function') {
		return elem[functionName].call(elem);
	}
};

SemwidgJS.Util.setValue = function(id, propertyName, value) {

	var elem = SemwidgJS.Util.getSemwidgElementById(id);
	var functionName = 'set' + SemwidgJS.Util.String.toUpperCamelCase(propertyName);

	if (elem && typeof elem[functionName] === 'function') {
		elem[functionName].call(elem, value);
		return elem;
	}
};

SemwidgJS.Util.getClass = function(obj) {
	if (typeof obj === 'undefined') {
		return 'undefined';
	}

	if (obj === null) {
		return 'null';
	}

	return Object.prototype.toString.call(obj).match(/^\[object\s(.*)\]$/)[1];
};

SemwidgJS.Util.getDefaultProxy = function() {
	return SemwidgJS.getSourceLocation() + 'TODO/DefineProxyPath.php';
};

SemwidgJS.Util.destroy = function(semwidgElement) {
	if (semwidgElement instanceof SemwidgJS.SwWidget) {
		SemwidgJS.Util._destroyWidget(semwidgElement);
	}else if(semwidgElement instanceof SemwidgJS.WidgetProvider) {
		SemwidgJS.Util._destroyWidgetProvider(semwidgElement);
	}else if(semwidgElement instanceof SemwidgJS.ConfigElement) {
		SemwidgJS.Util._destroyConfigElement(semwidgElement);
	}else if(semwidgElement instanceof SemwidgJS.Config) {
		SemwidgJS.Util._destroyConfig(semwidgElement);
	}else if(semwidgElement instanceof SemwidgJS.Template) {
		SemwidgJS.Util._destroyTemplate(semwidgElement);
	}else if(semwidgElement instanceof SemwidgJS.TemplateDefinition) {
		SemwidgJS.Util._destroyTemplateDefinition(semwidgElement);
	}else if(semwidgElement instanceof SemwidgJS.TemplateProvider) {
		SemwidgJS.Util._destroyTemplateProvider(semwidgElement);
	}else {
		console.warn('TODO: implement destroy function');
		console.warn(semwidgElement);
	}
};

SemwidgJS.Util._destroyWidget = function(widget) {

	for (var property in widget.parameterObjects) {
		var parameterObject = widget.parameterObjects[property];
		for (var i = 0; i < parameterObject.parameterObjectParts.length; i++) {
			var pop = parameterObject.parameterObjectParts[i];

			SemwidgJS.EventDispatcher._removeAllReferences(pop);
			pop.setNamedResources(null);
			pop.setParameterObject(null);
			pop.setNamedResources(null);
			pop.setSparqlResult(null);
			parameterObject.parameterObjectParts[i] = null;
		}
		SemwidgJS.EventDispatcher._removeAllReferences(parameterObject);
		parameterObject.setWidget(null);
		widget.parameterObjects[property] = null;
	}
	widget._destroy();
	$(widget._getNode()).html('');
	widget._setNode(null);
	widget.owner = null;
	SemwidgJS.EventDispatcher._removeAllReferences(widget);
};

SemwidgJS.Util._destroyWidgetProvider = function(widgetProvider) {

	var widgets = widgetProvider.getWidgets();
	for (var i = widgets.length - 1; i >= 0; i--) {
		widgetProvider.removeWidget(widgets[i]);
	}
	SemwidgJS.EventDispatcher._removeAllReferences(widgetProvider);

};

SemwidgJS.Util._destroyConfigElement = function(configElement) {
	SemwidgJS.EventDispatcher._removeAllReferences(configElement);
	configElement._setNode(null);
};

SemwidgJS.Util._destroyConfig = function(config) {
	
	config.clear();

	SemwidgJS.EventDispatcher._removeAllReferences(config);
};

SemwidgJS.Util._destroyTemplate = function(template) {
	template.removeWidgets();
	SemwidgJS.EventDispatcher._removeAllReferences(template);
	$(template._getNode()).html('');
	template._setNode(null);
};

SemwidgJS.Util._destroyTemplateDefinition = function(templateDefinition) {

	SemwidgJS.EventDispatcher._removeAllReferences(templateDefinition);
	templateDefinition._setNode(null);
};

SemwidgJS.Util._destroyTemplateProvider = function(templateProvider) {

	var templates = templateProvider.getTemplates();
	for (var i = templates.length - 1; i >= 0; i--) {
		templateProvider.removeTemplate(templates[i]);
	}

	var templateDefinitions = templateProvider.getTemplateDefinitions();
	for (var i = templateDefinitions.length - 1; i >= 0; i--) {
		templateProvider.removeTemplateDefinition(templateDefinitions[i]);
	}

	SemwidgJS.EventDispatcher._removeAllReferences(templateProvider);

};

SemwidgJS.Util.addJS = function(src, id, onReady) {

	var context = window.document;

	try {

		var add = true;

		if (id) {
			add = ($(context.head).find('#' + id).length >= 1) ? false : true;
		}
		
		if (add) {
			var scriptElement = context.createElement('script');

			if (id) {
				scriptElement.id = id;
			}

			scriptElement.type = 'text/javascript';
			scriptElement.src = src;

			if (onReady) {
				scriptElement.onload = onReady;						//most browsers
				scriptElement.onreadystatechange = function() {		//ie
					if (this.readyState == 'complete') {
						onReady();
					}
				}
			}

			context.head.appendChild(scriptElement);

		}else {
			onReady();
		}

	}catch (e) {
		if (SemwidgJS.showExceptions) {
			console.error(e);
		}
	}	

};

SemwidgJS.Util.addCSS = function(src, id, onReady) {

	var context = window.document;

	try {

		var add = true;

		if (id) {
			add = ($(context.head).find('#' + id).length >= 1) ? false : true;
		}
		
		if (add) {
			var styleElement = context.createElement('link');

			if (id) {
				styleElement.id = id;
			}

			styleElement.rel = 'stylesheet';
			styleElement.type = 'text/css';
			styleElement.href = src;

			if (onReady) {
				styleElement.onload = onReady;						//most browsers
				styleElement.onreadystatechange = function() {		//ie
					if (this.readyState == 'complete') {
						onReady();
					}
				}
			}

			context.head.appendChild(styleElement);

		}else {
			onReady();
		}

	}catch (e) {
		if (SemwidgJS.showExceptions) {
			console.error(e);
		}
	}	

};



SemwidgJS.Util.String = function() {

};

/**
 * Checks if a String is undefined, null, false, 0, empty, or blank (whitespaces)
 */
SemwidgJS.Util.String.isEmpty = function(str) {
	return (!str || str.length === 0 || /^\s*$/.test(str));
};

SemwidgJS.Util.String.replaceAll = function(str, find, replace) {
	
	if (find == '.') {
		find = '\\.';
	}

	return str.replace(new RegExp(find, 'g'), replace);
};

SemwidgJS.Util.String.htmlEscape = function(str) {
	str = '' + str;

	var entityMap = {
		'&': '&amp;',
		'<': '&lt;',
		'>': '&gt;',
		'"': '&quot;',
		"'": '&#39;',
		'/': '&#x2F;'
	};

	return str.replace(/[&<>"'\/]/g, function (s) {
		return entityMap[s];
	});

};

SemwidgJS.Util.String.htmlUnescape = function(str) {
	str = '' + str;

	str = SemwidgJS.Util.String.replaceAll(str, '&amp;', '&');
	str = SemwidgJS.Util.String.replaceAll(str, '&lt;', '<');
	str = SemwidgJS.Util.String.replaceAll(str, '&gt;', '>');
	str = SemwidgJS.Util.String.replaceAll(str, '&quot;', '"');
	str = SemwidgJS.Util.String.replaceAll(str, '&#39;', "'");
	str = SemwidgJS.Util.String.replaceAll(str, '&#x2F;', '/');
	
	return str;
};

SemwidgJS.Util.String.startsWith = function (haystack, needle) {
	return haystack.slice(0, needle.length) == needle;
};

SemwidgJS.Util.String.endsWith = function(haystack, needle) {
    return haystack.indexOf(needle, haystack.length - needle.length) !== -1;
};

SemwidgJS.Util.String.getIndicesOf = function(haystack, needle, caseSensitive) {
    
	var startIndex = 0;
	var searchStrLen = needle.length;
	var index = [];
	var indices = [];

	if (!caseSensitive) {
		haystack = haystack.toLowerCase();
		needle = needle.toLowerCase();
	}

	while ((index = haystack.indexOf(needle, startIndex)) >= 0) {
		indices.push(index);
		startIndex = index + searchStrLen;
	}

	if (indices.length > 0) {
		return indices;
	}

	return null;
};

SemwidgJS.Util.String.toCamelCase = function(str) {

	if (typeof str !== 'string') {
		return;
	}

	str = str.replace(/([^a-zA-Z0-9_\- ])|^[_0-9]+/g, '').trim().toLowerCase();
	str = str.replace(/([ -]+)([a-zA-Z0-9])/g, function(a, b, c) {
		return c.toUpperCase();
	});
	str = str.replace(/([0-9]+)([a-zA-Z])/g, function(a, b, c) {
		return b + c.toUpperCase();
	});
	return str;
};

SemwidgJS.Util.String.toUpperCamelCase = function(str) {
	return SemwidgJS.Util.String.toCamelCase('-' + str);
};

SemwidgJS.Util.Widget = function() {

};

SemwidgJS.Util.Widget.getWidgetClassFromString = function(name) {

	if (!SemwidgJS.Util.String.isEmpty(name)) {

		var path = name.split('.');
		var parent = (window || global || this);

		for (var i = 0; i < path.length; i++) {
			parent = parent[path[i]];
		}

		if (typeof parent !== 'function') {
			console.error('Widget class ' + name + ' not found.');
		}

		return parent;
	}

};

SemwidgJS.Util.Widget.createWidgetTypeFunction = function(widgetClass, name) {

	if (!widgetClass.prototype.hasOwnProperty('_getType')) {
		widgetClass.prototype['_getType'] = function() {
			return name;
		};
	}
};

// Creates getter and setter functions
SemwidgJS.Util.Widget.createWidgetPropertyFunctions = function(widgetClass, property) {
	
	var propertyNameCapital = SemwidgJS.Util.String.toUpperCamelCase(property[0]);
	var propertyName = SemwidgJS.Util.String.toCamelCase(property[0]);

	var type = property[1].toLowerCase();
	var functionName;

	// Property getter/setter
	functionName = 'get' + propertyNameCapital;
	if (typeof widgetClass.prototype[functionName] !== 'function') {
		widgetClass.prototype[functionName] = function() {
			return this[propertyName];
		};
	}

	functionName = 'set' + propertyNameCapital;
	if (typeof widgetClass.prototype[functionName] !== 'function') {
		widgetClass.prototype[functionName] = function(value) {

			this[propertyName] = value;
			
			if (this._getProperties()[propertyNameCapital] && 
				this._getProperties()[propertyNameCapital][1] == 'query') {
				this._addParameterObject(propertyNameCapital, value);
			}

			this._dispatchEvent(new SemwidgJS.Event(propertyName + 'Changed', value));
			this._dispatchEvent(new SemwidgJS.Event('semwidgJsPropertyChanged', propertyName));
		};
	}

	functionName = 'get' + propertyNameCapital + 'PropertyDescription';
	if (typeof widgetClass.prototype[functionName] !== 'function') {
		widgetClass.prototype[functionName] = function() {
			return property;
		};
	}

	// Endpoint, Order, Selector, Transform, Refresh, Limit getter/setter
	if (type == 'query') {

		var queryProperties = widgetClass.prototype._getQueryProperties();

		for (var i = 0; i < queryProperties.length; i++) {
			(function(propertyNameCapital, queryProperty) {

				var functionSuffix = SemwidgJS.Util.String.toUpperCamelCase(queryProperty[0]);

				functionName = 'get' + propertyNameCapital + functionSuffix;
				if (typeof widgetClass.prototype[functionName] !== 'function') {
					widgetClass.prototype[functionName] = function() {
						return this[propertyName + functionSuffix];
					};
				}

				functionName = 'set' + propertyNameCapital + functionSuffix;
				if (typeof widgetClass.prototype[functionName] !== 'function') {
					widgetClass.prototype[functionName] = function(value) {
						this[propertyName + functionSuffix] = value;
						this._dispatchEvent(new SemwidgJS.Event(propertyName + functionSuffix + 'Changed', value));
						this._dispatchEvent(new SemwidgJS.Event('semwidgJsPropertyChanged', propertyName + functionSuffix));
					};
				}

				functionName = 'get' + propertyNameCapital + functionSuffix + 'PropertyDescription';
				if (typeof widgetClass.prototype[functionName] !== 'function') {
					widgetClass.prototype[functionName] = function() {
						return queryProperty;
					};
				}
			}(propertyNameCapital, queryProperties[i]));
		}

	}

	// Default value getter
	functionName = 'get' + propertyNameCapital + 'DefaultValue';
	if (typeof widgetClass.prototype[functionName] !== 'function') {

		if (type == 'query') {
			widgetClass.prototype[functionName] = function() {
				
				if (this._getParameterObjects()) {
					var parameterObject = this._getParameterObjects()[propertyNameCapital];
					if (parameterObject) {
						var selectorFunctionName = 'get' + propertyNameCapital + 'Selector';
						var selectorFunctionValue = this[selectorFunctionName]();
						var selectorFunction;
						
						if (selectorFunctionValue) {
							selectorFunctionValue = ('' + selectorFunctionValue).trim();

							if (SemwidgJS.Util.String.startsWith(selectorFunctionValue, 'return')) {
								selectorFunction = new Function('data', selectorFunctionValue);
							}else if(window[selectorFunctionValue]){
								selectorFunction = window[selectorFunctionValue];
							}
						}
						
						return parameterObject.getDefaultValue(selectorFunction);
					}
				}
			};
		}else {
			widgetClass.prototype[functionName] = function() {
				return this[propertyName];
			};
		}
	}

	// All values getter
	functionName = 'get' + propertyNameCapital + 'AllValues';
	if (typeof widgetClass.prototype[functionName] !== 'function') {

		if (type == 'query') {
			widgetClass.prototype[functionName] = function() {
				
				var allValues = [];
				if (this._getParameterObjects()) {
					var parameterObject = this._getParameterObjects()[propertyNameCapital];
					if (parameterObject) {
						var parts = parameterObject.getParameterObjectParts();
						for (var i = 0; i < parts.length; i++) {
							// console.log(parts);
							if (parts[i] && parts[i].results && parts[i].results[0] && parts[i].results[0].resultObject) {

								allValues.push(parts[i].results[0].resultObject);
							}
						}
					}
				}
				return allValues;
				
			};
		}else {
			widgetClass.prototype[functionName] = function() {
				return this[propertyName];
			};
		}
	}

	functionName = 'get' + propertyNameCapital + 'JoinedValues';
	if (typeof widgetClass.prototype[functionName] !== 'function') {
		if (type == 'query') {
			widgetClass.prototype[functionName] = function() {
				
				if (this._getParameterObjects()) {
					var parameterObject = this._getParameterObjects()[propertyNameCapital];

					var joinedValues = [];
					
					if (parameterObject) {

						var parts;
						parts = parameterObject.getParameterObjectParts();

						if (parts.length < parameterObject.getOriginalStringArray().length) {
							return joinedValues;
						}

						var mergeResultObjects = function(obj1, obj2) {

							var merged = new SemwidgJS.SparqlResult.ResultObject();
							merged.variableNames = obj1.variableNames.concat(obj2.variableNames);
							merged.results = [];

							for (var i = 0; i < obj1.results.length; i++) {
								for (var j = 0; j < obj2.results.length; j++) {
									merged.results.push(obj1.results[i].concat(obj2.results[j]));
								}
							}

							return merged;

						};

						var merge = function(jv, res) {

							var tmp = [];

							if (!jv || jv.length == 0) {
								tmp.push(res);
								return tmp;
							}

							if (!res) {
								tmp = jv;
								return tmp;
							}

							var i, j = 0;
							var merged;
							for (i = 0; i < jv.length; i++) {

								merged = mergeResultObjects(jv[i], res);
								tmp.push(merged);
							}

							return tmp;
						};

						var part;
						var results;
						var resultObject;
						var index = 0;
						var tempJoinedValues;
						for (var i = 0; i < parts.length; i++) {

							part = parts[i];

							if (part.type == 'string') {
								resultObject = new SemwidgJS.SparqlResult.ResultObject();
								resultObject.variableNames = ['string'];

								var binding = new SemwidgJS.SparqlResult.ResultBinding();
								binding.name = 'string';
								binding.type = 'literal';
								binding.value = part.getOriginalString();

								resultObject.results = [[binding]];

								tempJoinedValues = [];
								if (joinedValues.length == 0) {
									tempJoinedValues.push(merge(null, resultObject));
								}else {

									for (var k = 0; k < joinedValues.length; k++) {
										tempJoinedValues.push(merge(joinedValues[k], resultObject));
									}
								}
								joinedValues = tempJoinedValues;

							}else {

								var j;
								var emptyResults = false;
								for (j = 0; j < part.getSparqlResult().length; j++) {
									if (!part.getSparqlResult()[j]) {
										emptyResults = true;
										break;
									}
								}

								if (emptyResults) {
									continue;
								}

								tempJoinedValues = [];
								for (j = 0; j < part.getSparqlResult().length; j++) {
									resultObject = part.getSparqlResult()[j].resultObject;
									if (joinedValues.length == 0) {
										tempJoinedValues.push(merge(null, resultObject));
									}else {
										for (var k = 0; k < joinedValues.length; k++) {
											tempJoinedValues.push(merge(joinedValues[k], resultObject));
										}
									}
								}
								joinedValues = tempJoinedValues;

							}
						}
					}

					return joinedValues;
				}
			};
		}else {
			widgetClass.prototype[functionName] = function() {
				return this[propertyName];
			};
		}
	}
};

SemwidgJS.Util.Widget.parseParameterParts = function(value) {
	
	var parts = [];

	var counter = 0;
	var previous = 0;
	var lastZeroIndex = 0;
	var i;

	for (i = 0; i < value.length; i++) {

		if (counter == 0 && previous == 1 && (i - lastZeroIndex) > 0) {
			parts.push(value.substring(lastZeroIndex, i));
			lastZeroIndex = i;
		}else if (counter == 1 && previous == 0 && (i - 1 - lastZeroIndex) > 0) {
			parts.push(value.substring(lastZeroIndex, i - 1));
			lastZeroIndex = i - 1;
		}

		previous = counter;

		if (value[i] == '{') {
			counter++;
		}else if(value[i] == '}' && counter > 0) {
			counter--;
		}
	}

	parts.push(value.substring(lastZeroIndex, i));

	return parts;
};

SemwidgJS.Util.SemwidgQL = function() {

};

SemwidgJS.Util.SemwidgQL.toSparql = function(semwidgQuery, limit, throwExceptions) {

	var sparqlQueries;

	var resources = SemwidgJS.Util.SemwidgQL.convertNamedResources(
			SemwidgJS.getConfig().getResources());

	var prefixes = SemwidgJS.Util.SemwidgQL.convertPrefixes(
			SemwidgJS.getConfig().getNamespaces());
	
	sparqlQueries = SemwidgQL.toSparql(semwidgQuery, prefixes, resources, throwExceptions);

	if (sparqlQueries) {

		for (var i = 0; i < sparqlQueries.length; i++) {
			if (!limit) {
				sparqlQueries[i] += '\nLIMIT 100';
			}else if(limit > 0) {
				sparqlQueries[i] += '\nLIMIT ' + limit;
			}
		}
	}

	return sparqlQueries;
};

SemwidgJS.Util.SemwidgQL.convertNamedResources = function(namedResourceSemwidgJS) {

	var namedResourceSemwidgQL = new Array();

	for (var i = 0; i < namedResourceSemwidgJS.length; i++) {
		namedResourceSemwidgQL.push(
				new SemwidgQL.NamedResource(namedResourceSemwidgJS[i].getId(), namedResourceSemwidgJS[i].getUri())
			);
	}

	return namedResourceSemwidgQL;

};

SemwidgJS.Util.SemwidgQL.convertPrefixes = function(prefixesSemwidgJS) {

	var prefixesSemwidgQL = new Array();

	for (var i = 0; i < prefixesSemwidgJS.length; i++) {
		prefixesSemwidgQL.push(
				new SemwidgQL.Prefix(prefixesSemwidgJS[i].getId(), prefixesSemwidgJS[i].getUri())
			);
	}

	return prefixesSemwidgQL;

};

//+ Carlos R. L. Rodrigues
//@ http://jsfromhell.com/classes/math-parser [rev. #3]
SemwidgJS.Util.MathParser = function() {
    var o = this, p = o.operator = {};
    p["+"] = function(n, m){return n + m;}
    p["-"] = function(n, m){return n - m;}
    p["*"] = function(n, m){return n * m;}
    p["/"] = function(m, n){return n / m;}
    p["%"] = function(m, n){return n % m;}
    p["^"] = function(m, n){return Math.pow(n, m);}
    p["~"] = function(m, n){return Math.sqrt(n, m);}
    o.custom = {}, p.f = function(s, n){
        if(Math[s]) return Math[s](n);
        else if(o.custom[s]) return o.custom[s].apply(o, n);
        else throw new Error("Function \"" + s + "\" not defined.");
    }, o.add = function(n, f){this.custom[n] = f;}
};

SemwidgJS.Util.MathParser.prototype.eval = function(e, ig){
    var v = [], p = [], i, l, _, a, c = 0, s = 0, x, t = !ig ? e.indexOf("^") : -1, d = null;
    var cp = e, e = e.split(""), n = "0123456789.", o = "+-/*^%~", f = this.operator;
    if(t + 1)
        do{
            for(a = "", _ = t - 1;  _ && o.indexOf(e[_]) < 0; a += e[_], e[_--] = ""); a += "^";
            for(_ = t + 1, i = e.length; _ < i && o.indexOf(e[_]) < 0; a += e[_], e[_++] = "");
            e = e.slice(0, t).concat((this.eval(a, 1) + "").split("")).concat(e.slice(t + 1));
        }
        while(t = cp.indexOf("^", ++t) + 1);
    for(i = 0, l = e.length; i < l; i++){
        if(o.indexOf(e[i]) > -1)
            e[i] == "-" && (s > 1 || d === null) && ++s, !s && d !== null && (p.push(e[i]), s = 2), "+-".indexOf(e[i]) < (d = null) && (c = 1);
        else if(a = n.indexOf(e[i]) + 1 ? e[i++] : ""){
            while(n.indexOf(e[i]) + 1) a += e[i++];
            v.push(d = (s & 1 ? -1 : 1) * a), c && v.push(f[p.pop()](v.pop(), v.pop())) && (c = 0), --i, s = 0;
        }
    }
    for(c = v[0], i = 0, l = p.length; l--; c = f[p[i]](c, v[++i]));
    return c;
};

SemwidgJS.Util.MathParser.prototype.parse = function(e){
    var p = [], f = [], ag, n, c, a, o = this, v = "0123456789.+-*/^%~(, )";
    for(var x, i = 0, l = e.length; i < l; i++){
        if(v.indexOf(c = e.charAt(i)) < 0){
            for(a = c; v.indexOf(c = e.charAt(++i)) < 0; a += c); f.push((--i, a));
        }
        else if(!(c == "(" && p.push(i)) && c == ")"){
            if(a = e.slice(0, (n = p.pop()) - (x = v.indexOf(e.charAt(n - 1)) < 0 ? y = (c = f.pop()).length : 0)), x)
                for(var j = (ag = e.slice(n, ++i).split(",")).length; j--; ag[j] = o.eval(ag[j]));
            l = (e = a + (x ? o.operator.f(c, ag) : o.eval(e.slice(n, ++i))) + e.slice(i)).length, i -= i - n + c.length;
        }
    }
    return o.eval(e);
};

SemwidgJS.Util._colors = [
	'EF7C00',
	'BD1789',
	'86BA0C',
	'0F16BA',
	'0A8566',
	'EF450E',
	'9D14BD',
	'BA9B14',
	'0D6FBA',
	'0B8511',
	'EF4C5A',
	'564ABD',
	'BA844B',
	'3EBAA3',
	'75853D',
	'D7A1EF',
	'BA7D76',
	'759FBD',
	'E0D8A3',
	'74BA86'
];

SemwidgJS.Util.getColors = function() {
	return SemwidgJS.Util._colors;
};

SemwidgJS.Util.getColor = function(index) {
	return SemwidgJS.Util._colors[(index % SemwidgJS.Util._colors.length)];
};

SemwidgJS.Util.shadeColor = function(color, percent) {
	var f = parseInt(color.slice(1), 16);
	var t = (percent < 0) ? 0: 255;
	var p = (percent < 0) ? percent * -1 : percent;
	var R = f >> 16;
	var G = f >> 8 & 0x00FF;
	var B = f & 0x0000FF;
	return '#' + (0x1000000 + 
			(Math.round((t - R) * p) + R) * 0x10000 + 
			(Math.round((t - G) * p) + G) * 0x100 + 
			(Math.round((t - B) * p) + B)).toString(16).slice(1);
};

SemwidgJS.Util.getBrightness = function(color) {
	var f = parseInt(color.slice(1), 16);
	var R = f >> 16;
	var G = f >> 8 & 0x00FF;
	var B = f & 0x0000FF;

	return 0.2126 * R + 0.7152 * G + 0.0722 * B; //ITU-R BT.709
};

SemwidgJS.Util.sanitizeHtml = function(value) {
	if (!value) {
		return '';
	}
	var output = value.replace(/<script[^>]*?>.*?<\/script>/gi, '').
				 replace(/<style[^>]*?>.*?<\/style>/gi, '').
				 replace(/<![\s\S]*?--[ \t\n\r]*>/gi, '');
	return output;
};