/**
 * Semwidg Widget Provider
 * 
 * stores all widgets and it provides widget related functions
 */

SemwidgJS.WidgetProvider = function() {

};

SemwidgJS.WidgetProvider.prototype = new SemwidgJS.EventDispatcher();

SemwidgJS.WidgetProvider.prototype.addWidget = function(widget) {
	
	if (!SemwidgJS.Util._checkIdWidget(widget.getId())) {
	// 	var newId = SemwidgJS.Util.String.replaceAll(widget.getId(), '.', '') + '-' + SemwidgJS.Util.getUniqueId();
		
		console.warn('Id "' + widget.getId() + '" is already defined or contains an unallowed character.');

	// 	widget.setId(newId, true);
	// 	widget._writeToDom('id');
	}

	if (widget) {
		var w = this.getWidget(widget.getId());
		if (w) {
			this.removeWidget(w);
		}
		
		this.getWidgets().push(widget);
	}

	
	
	this._dispatchEvent(new SemwidgJS.Event('widgetAdded', widget));
};

SemwidgJS.WidgetProvider.prototype.getWidget = function(id) {
	for (var i = 0; i < this.getWidgets().length; i++) {
		if (this.getWidgets()[i].getId() == id) {
			return this.getWidgets()[i];
		}
	}
	return null;
};

SemwidgJS.WidgetProvider.prototype.getWidgetIndex = function(widget) {
	for (var i = 0; i < this.getWidgets().length; i++) {
		if (this.getWidgets()[i].getId() == widget.getId()) {
			return i;
		}
	}
	return null;
};

SemwidgJS.WidgetProvider.prototype.getWidgets = function() {
	if (!this.widgets) {
		this.widgets = [];
	}
	return this.widgets;
};

SemwidgJS.WidgetProvider.prototype.removeWidget = function(widget) {
	var widgets = this.getWidgets();

	SemwidgJS.Util.destroy(widget);

	var index = -1;

	for (var i = 0; i < widgets.length; i++) {
		if (widgets[i] == widget) {
			index = i;
			break;
		}
	}

	widgets[i] = null;

	if (index > -1) {
		widgets.splice(index, 1);
	}
};

SemwidgJS.WidgetProvider.prototype.readWidgets = function(node) {

	var widgetNodeList;

	if (node) {
		widgetNodeList = $(node).find('[data-sw-widget]');
	}else {
		widgetNodeList = $.find('[data-sw-widget]');
	}

	for (var i = 0; i < widgetNodeList.length; i++) {

		var widgetNode = widgetNodeList[i];

		var templateDefinitionParent = $(widgetNode).closest('[data-sw-config="template"]');

		// Only process widgets outside of TemplateDefinitions
		if (templateDefinitionParent.length == 0) {

			// If widget was already defined -> delete
			var id = $(widgetNode).attr('data-swp-id');
			var widget = this.getWidget(id);

			if (widget) {
				this.removeWidget(widget);
			}

			widget = this.buildWidgetFromNode(widgetNode);
			this.addWidget(widget);
			widget._build();
		}
	}
};

SemwidgJS.WidgetProvider.prototype.buildWidgetFromNode = function(node) {

	var widget;
	var objNode = $(node);

	var widgetType = objNode.data('sw-widget');

	if (SemwidgJS.WidgetProvider.getWidgetTypes().hasOwnProperty(widgetType)) {

		var widgetClass = SemwidgJS.WidgetProvider.getWidgetTypes()[widgetType];
		widget = new widgetClass();

	}else {
		console.error('No component registered for widget of type "' + widgetType + '"');
		widget = new SemwidgJS.SwWidget();
	}
	
	widget._setNode(node);

	// Is Widget descendant of a Template?
	var template = objNode.closest('[data-sw-template]');
	var prefix = template.data('swp-id');

	// First: set id!
	var id = objNode.data('swp-id');
	if (SemwidgJS.Util.String.isEmpty(id)) {
		widget._generateId(prefix);
		widget._writeToDom('id');
	}else {
		if (!SemwidgJS.Util.String.isEmpty(prefix)) {
			id = prefix + ':' + id;
		}
		widget.setId(id);
		if (!SemwidgJS.Util.String.isEmpty(prefix)) {
			widget._writeToDom('id');
		}
	}

	var dataAttributes = objNode.data();

	var queryAttributes = [];
	var otherAttributes = [];

	for (var dataAttribute in dataAttributes) {
		
		if (SemwidgJS.Util.String.startsWith(dataAttribute, 'swp')) {
			
			var propertyName = dataAttribute.slice(3);
			var functionName = 'get' + propertyName + 'PropertyDescription';
			
			if (typeof widget[functionName] === 'function') {

				var description = widget[functionName].call(widget);
				
				var value = SemwidgJS.Util.String.htmlUnescape(objNode.data(dataAttribute)).trim();

				if (value != '') {
					if (description[1] == 'query') {
						queryAttributes.push([propertyName, value]);
					}else if (propertyName != 'Id') {
						otherAttributes.push([propertyName, value]);
					}
				}
			}
		}
	}

	var i = 0;
	for (i = 0; i < otherAttributes.length; i++) {
		var functionName = 'set' + otherAttributes[i][0];
		if (typeof widget[functionName] === 'function') {
			widget[functionName].call(widget, otherAttributes[i][1]);
		}else {
			console.warn('Attribute "' + otherAttributes[i][0] 
				 + '" is not available for widget type "' + widgetType + '"');
		}
	}

	for (i = 0; i < queryAttributes.length; i++) {
		var functionName = 'set' + queryAttributes[i][0];
		if (typeof widget[functionName] === 'function') {
			widget[functionName].call(widget, queryAttributes[i][1]);
		}else {
			console.warn('Attribute "' + queryAttributes[i][0] 
				 + '" is not available for widget type "' + widgetType + '"');
		}
	}

	widget._addEventListener('semwidgJsPropertyChanged', widget._build, widget);

	return widget;
};


SemwidgJS.WidgetProvider.getWidgetTypes = function() {
	if (!SemwidgJS.WidgetProvider.widgetTypes) {
		SemwidgJS.WidgetProvider.widgetTypes = {};
	}
	return SemwidgJS.WidgetProvider.widgetTypes;
};

SemwidgJS.WidgetProvider.registerWidgetType = function(name) {

	var widget = SemwidgJS.Util.Widget.getWidgetClassFromString(name);

	if (widget) {

		var widgetTypes = SemwidgJS.WidgetProvider.getWidgetTypes();
		var widgetName = name.split('.').pop();
		var widgetInstance = new widget();

		SemwidgJS.Util.Widget.createWidgetTypeFunction(widget, widgetName);

		var properties = widgetInstance._getProperties();

		for (var property in properties) {
			SemwidgJS.Util.Widget.createWidgetPropertyFunctions(widget, properties[property]);
		}

		if (widgetTypes.hasOwnProperty(widgetName)) {
			console.warn('Warning: Overriding component for widget type "'  + widgetName + '"');
		}
		
		widgetTypes[widgetName] = widget;
	}

};
